@@ -1,9 +1,11 @@
-/* ====================================================================
-   Copyright 2004   Apache Software Foundation
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+/* ====================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -13,7 +15,7 @@
    See the License for the specific language governing permissions and
    limitations under the License.
 ==================================================================== */
-
+        
 package org.apache.poi.ddf;
 
 /**
@@ -27,7 +29,7 @@ public class EscherRGBProperty
 
     public EscherRGBProperty( short propertyNumber, int rgbColor )
     {
-        super( propertyNumber, false, false, rgbColor );
+        super( propertyNumber, rgbColor );
     }
 
     public int getRgbColor()
