@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -13,13 +14,16 @@
    See the License for the specific language governing permissions and
    limitations under the License.
 ==================================================================== */
-
+        
 package org.apache.poi.hpsf;
 
-import java.io.*;
-import java.util.*;
-import org.apache.poi.hpsf.wellknown.*;
-import org.apache.poi.poifs.filesystem.*;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.UnsupportedEncodingException;
+import java.util.ArrayList;
+import java.util.List;
+
+import org.apache.poi.hpsf.wellknown.SectionIDMap;
 import org.apache.poi.util.LittleEndian;
 
 /**
@@ -50,9 +54,10 @@ import org.apache.poi.util.LittleEndian;
  * NoSingleSectionException} if the {@link PropertySet} contains more
  * (or less) than exactly one {@link Section}).</p>
  *
- * @author Rainer Klute (klute@rainer-klute.de)
+ * @author Rainer Klute <a
+ * href="mailto:klute@rainer-klute.de">&lt;klute@rainer-klute.de&gt;</a>
  * @author Drew Varner (Drew.Varner hanginIn sc.edu)
- * @version $Id: PropertySet.java,v 1.9.4.1 2004/02/22 11:54:45 glens Exp $
+ * @version $Id: PropertySet.java 489730 2006-12-22 19:18:16Z bayard $
  * @since 2002-02-09
  */
 public class PropertySet
@@ -61,8 +66,8 @@ public class PropertySet
     /**
      * <p>The "byteOrder" field must equal this value.</p>
      */
-    final static byte[] BYTE_ORDER_ASSERTION =
-	new byte[]{(byte) 0xFE, (byte) 0xFF};
+    static final byte[] BYTE_ORDER_ASSERTION =
+        new byte[] {(byte) 0xFE, (byte) 0xFF};
 
     /**
      * <p>Specifies this {@link PropertySet}'s byte order. See the
@@ -86,8 +91,8 @@ public class PropertySet
     /**
      * <p>The "format" field must equal this value.</p>
      */
-    final static byte[] FORMAT_ASSERTION =
-	new byte[]{(byte) 0x00, (byte) 0x00};
+    static final byte[] FORMAT_ASSERTION =
+        new byte[]{(byte) 0x00, (byte) 0x00};
 
     /**
      * <p>Specifies this {@link PropertySet}'s format. See the HPFS
@@ -116,21 +121,31 @@ public class PropertySet
     protected int osVersion;
 
 
-    public final static int OS_WIN16     = 0x0000;
-    public final static int OS_MACINTOSH = 0x0001;
-    public final static int OS_WIN32     = 0x0002;
+    /**
+     * <p>If the OS version field holds this value the property set stream was
+     * created on a 16-bit Windows system.</p>
+     */
+    public static final int OS_WIN16     = 0x0000;
+
+    /**
+     * <p>If the OS version field holds this value the property set stream was
+     * created on a Macintosh system.</p>
+     */
+    public static final int OS_MACINTOSH = 0x0001;
+
+    /**
+     * <p>If the OS version field holds this value the property set stream was
+     * created on a 32-bit Windows system.</p>
+     */
+    public static final int OS_WIN32     = 0x0002;
 
     /**
      * <p>Returns the property set stream's low-level "OS version"
      * field.</p>
      *
-     * <p><strong>FIXME:</strong> Return an <code>int</code> instead
-     * of a <code>long</code> in the next major version, i.e. when
-     * incompatible changes are allowed.</p>
-     *
      * @return The property set stream's low-level "OS version" field.
      */
-    public long getOSVersion()
+    public int getOSVersion()
     {
         return osVersion;
     }
@@ -156,25 +171,15 @@ public class PropertySet
 
 
 
-    /**
-     * <p>The number of sections in this {@link PropertySet}.</p>
-     */
-    protected int sectionCount;
-
-
     /**
      * <p>Returns the number of {@link Section}s in the property
      * set.</p>
      *
-     * <p><strong>FIXME:</strong> Return an <code>int</code> instead
-     * of a <code>long</code> in the next major version, i.e. when
-     * incompatible changes are allowed.</p>
-     *
      * @return The number of {@link Section}s in the property set.
      */
-    public long getSectionCount()
+    public int getSectionCount()
     {
-        return sectionCount;
+        return sections.size();
     }
 
 
@@ -206,7 +211,7 @@ public class PropertySet
      * writing functionality is implemented.</p>
      */
     protected PropertySet()
-    {}
+    { }
 
 
 
@@ -227,19 +232,23 @@ public class PropertySet
      * the {@link InputStream#markSupported} method.
      * @throws IOException if the {@link InputStream} cannot not be
      * accessed as needed.
+     * @exception NoPropertySetStreamException if the input stream does not
+     * contain a property set.
+     * @exception UnsupportedEncodingException if a character encoding is not
+     * supported.
      */
     public PropertySet(final InputStream stream)
-	throws NoPropertySetStreamException, MarkUnsupportedException,
-	       IOException
+        throws NoPropertySetStreamException, MarkUnsupportedException,
+               IOException, UnsupportedEncodingException
     {
         if (isPropertySetStream(stream))
-	{
+        {
             final int avail = stream.available();
             final byte[] buffer = new byte[avail];
             stream.read(buffer, 0, buffer.length);
             init(buffer, 0, buffer.length);
         }
-	else
+        else
             throw new NoPropertySetStreamException();
     }
 
@@ -257,13 +266,15 @@ public class PropertySet
      * @param length The length of the stream data.
      * @throws NoPropertySetStreamException if the byte array is not a
      * property set stream.
+     * 
+     * @exception UnsupportedEncodingException if the codepage is not supported.
      */
     public PropertySet(final byte[] stream, final int offset, final int length)
-	throws NoPropertySetStreamException
+        throws NoPropertySetStreamException, UnsupportedEncodingException
     {
         if (isPropertySetStream(stream, offset, length))
             init(stream, offset, length);
-	else
+        else
             throw new NoPropertySetStreamException();
     }
 
@@ -278,8 +289,11 @@ public class PropertySet
      * complete byte array contents is the stream data.
      * @throws NoPropertySetStreamException if the byte array is not a
      * property set stream.
+     * 
+     * @exception UnsupportedEncodingException if the codepage is not supported.
      */
-    public PropertySet(final byte[] stream) throws NoPropertySetStreamException
+    public PropertySet(final byte[] stream)
+    throws NoPropertySetStreamException, UnsupportedEncodingException
     {
         this(stream, 0, stream.length);
     }
@@ -299,9 +313,10 @@ public class PropertySet
      * stream, else <code>false</code>.
      * @throws MarkUnsupportedException if the {@link InputStream}
      * does not support the {@link InputStream#mark} method.
+     * @exception IOException if an I/O error occurs
      */
     public static boolean isPropertySetStream(final InputStream stream)
-	throws MarkUnsupportedException, IOException
+        throws MarkUnsupportedException, IOException
     {
         /*
          * Read at most this many bytes.
@@ -315,17 +330,17 @@ public class PropertySet
          */
         if (!stream.markSupported())
             throw new MarkUnsupportedException(stream.getClass().getName());
-	stream.mark(BUFFER_SIZE);
+        stream.mark(BUFFER_SIZE);
 
         /*
          * Read a couple of bytes from the stream.
          */
         final byte[] buffer = new byte[BUFFER_SIZE];
         final int bytes =
-	    stream.read(buffer, 0,
-			Math.min(buffer.length, stream.available()));
+            stream.read(buffer, 0,
+                        Math.min(buffer.length, stream.available()));
         final boolean isPropertySetStream =
-	    isPropertySetStream(buffer, 0, bytes);
+            isPropertySetStream(buffer, 0, bytes);
         stream.reset();
         return isPropertySetStream;
     }
@@ -343,34 +358,38 @@ public class PropertySet
      * @return <code>true</code> if the byte array is a property set
      * stream, <code>false</code> if not.
      */
-    public static boolean isPropertySetStream(final byte[] src, int offset,
-					      final int length)
+    public static boolean isPropertySetStream(final byte[] src,
+                                              final int offset,
+                                              final int length)
     {
+        /* FIXME (3): Ensure that at most "length" bytes are read. */
+
         /*
          * Read the header fields of the stream. They must always be
          * there.
          */
-        final int byteOrder = LittleEndian.getUShort(src, offset);
-        offset += LittleEndian.SHORT_SIZE;
+        int o = offset;
+        final int byteOrder = LittleEndian.getUShort(src, o);
+        o += LittleEndian.SHORT_SIZE;
         byte[] temp = new byte[LittleEndian.SHORT_SIZE];
-        LittleEndian.putShort(temp,(short)byteOrder);
+        LittleEndian.putShort(temp, (short) byteOrder);
         if (!Util.equal(temp, BYTE_ORDER_ASSERTION))
             return false;
-	final int format = LittleEndian.getUShort(src, offset);
-        offset += LittleEndian.SHORT_SIZE;
+        final int format = LittleEndian.getUShort(src, o);
+        o += LittleEndian.SHORT_SIZE;
         temp = new byte[LittleEndian.SHORT_SIZE];
-        LittleEndian.putShort(temp,(short)format);
+        LittleEndian.putShort(temp, (short) format);
         if (!Util.equal(temp, FORMAT_ASSERTION))
             return false;
-	final long osVersion = LittleEndian.getUInt(src, offset);
-        offset += LittleEndian.INT_SIZE;
-        final ClassID classID = new ClassID(src, offset);
-        offset += ClassID.LENGTH;
-        final long sectionCount = LittleEndian.getUInt(src, offset);
-        offset += LittleEndian.INT_SIZE;
+        // final long osVersion = LittleEndian.getUInt(src, offset);
+        o += LittleEndian.INT_SIZE;
+        // final ClassID classID = new ClassID(src, offset);
+        o += ClassID.LENGTH;
+        final long sectionCount = LittleEndian.getUInt(src, o);
+        o += LittleEndian.INT_SIZE;
         if (sectionCount < 1)
             return false;
-	return true;
+        return true;
     }
 
 
@@ -383,27 +402,33 @@ public class PropertySet
      *
      * @param src Byte array containing the property set stream
      * @param offset The property set stream starts at this offset
-     * from the beginning of <var>src</src>
+     * from the beginning of <var>src</var>
      * @param length Length of the property set stream.
+     * @throws UnsupportedEncodingException if HPSF does not (yet) support the
+     * property set's character encoding.
      */
-    private void init(final byte[] src, int offset, final int length)
+    private void init(final byte[] src, final int offset, final int length)
+    throws UnsupportedEncodingException
     {
+        /* FIXME (3): Ensure that at most "length" bytes are read. */
+        
         /*
          * Read the stream's header fields.
          */
-        byteOrder = LittleEndian.getUShort(src, offset);
-        offset += LittleEndian.SHORT_SIZE;
-        format = LittleEndian.getUShort(src, offset);
-        offset += LittleEndian.SHORT_SIZE;
-        osVersion = (int) LittleEndian.getUInt(src, offset);
-        offset += LittleEndian.INT_SIZE;
-        classID = new ClassID(src, offset);
-        offset += ClassID.LENGTH;
-        sectionCount = LittleEndian.getInt(src, offset);
-        offset += LittleEndian.INT_SIZE;
-	if (sectionCount <= 0)
-	    throw new HPSFRuntimeException("Section count " + sectionCount +
-					   " must be greater than 0.");
+        int o = offset;
+        byteOrder = LittleEndian.getUShort(src, o);
+        o += LittleEndian.SHORT_SIZE;
+        format = LittleEndian.getUShort(src, o);
+        o += LittleEndian.SHORT_SIZE;
+        osVersion = (int) LittleEndian.getUInt(src, o);
+        o += LittleEndian.INT_SIZE;
+        classID = new ClassID(src, o);
+        o += ClassID.LENGTH;
+        final int sectionCount = LittleEndian.getInt(src, o);
+        o += LittleEndian.INT_SIZE;
+        if (sectionCount <= 0)
+            throw new HPSFRuntimeException("Section count " + sectionCount +
+                                           " must be greater than 0.");
 
         /*
          * Read the sections, which are following the header. They
@@ -417,7 +442,7 @@ public class PropertySet
          * Summary Information stream has 2. Everything else is a rare
          * exception and is no longer fostered by Microsoft.
          */
-        sections = new ArrayList(2);
+        sections = new ArrayList(sectionCount);
 
         /*
          * Loop over the section descriptor array. Each descriptor
@@ -425,9 +450,9 @@ public class PropertySet
          * "offset" accordingly.
          */
         for (int i = 0; i < sectionCount; i++)
-	{
-            final Section s = new Section(src, offset);
-            offset += ClassID.LENGTH + LittleEndian.INT_SIZE;
+        {
+            final Section s = new Section(src, o);
+            o += ClassID.LENGTH + LittleEndian.INT_SIZE;
             sections.add(s);
         }
     }
@@ -444,7 +469,7 @@ public class PropertySet
     public boolean isSummaryInformation()
     {
         return Util.equal(((Section) sections.get(0)).getFormatID().getBytes(),
-			  SectionIDMap.SUMMARY_INFORMATION_ID);
+                          SectionIDMap.SUMMARY_INFORMATION_ID);
     }
 
 
@@ -459,7 +484,7 @@ public class PropertySet
     public boolean isDocumentSummaryInformation()
     {
         return Util.equal(((Section) sections.get(0)).getFormatID().getBytes(),
-			  SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID);
+                          SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID[0]);
     }
 
 
@@ -469,9 +494,7 @@ public class PropertySet
      * contained in this property set. It is a shortcut for getting
      * the {@link PropertySet}'s {@link Section}s list and then
      * getting the {@link Property} array from the first {@link
-     * Section}. However, it can only be used if the {@link
-     * PropertySet} contains exactly one {@link Section}, so check
-     * {@link #getSectionCount} first!</p>
+     * Section}.</p>
      *
      * @return The properties of the only {@link Section} of this
      * {@link PropertySet}.
@@ -479,9 +502,9 @@ public class PropertySet
      * more or less than one {@link Section}.
      */
     public Property[] getProperties()
-	throws NoSingleSectionException
+        throws NoSingleSectionException
     {
-        return getSingleSection().getProperties();
+        return getFirstSection().getProperties();
     }
 
 
@@ -499,7 +522,7 @@ public class PropertySet
      */
     protected Object getProperty(final int id) throws NoSingleSectionException
     {
-        return getSingleSection().getProperty(id);
+        return getFirstSection().getProperty(id);
     }
 
 
@@ -518,9 +541,9 @@ public class PropertySet
      * more or less than one {@link Section}.
      */
     protected boolean getPropertyBooleanValue(final int id)
-	throws NoSingleSectionException
+        throws NoSingleSectionException
     {
-        return getSingleSection().getPropertyBooleanValue(id);
+        return getFirstSection().getPropertyBooleanValue(id);
     }
 
 
@@ -538,9 +561,9 @@ public class PropertySet
      * more or less than one {@link Section}.
      */
     protected int getPropertyIntValue(final int id)
-	throws NoSingleSectionException
+        throws NoSingleSectionException
     {
-        return getSingleSection().getPropertyIntValue(id);
+        return getFirstSection().getPropertyIntValue(id);
     }
 
 
@@ -562,7 +585,21 @@ public class PropertySet
      */
     public boolean wasNull() throws NoSingleSectionException
     {
-        return getSingleSection().wasNull();
+        return getFirstSection().wasNull();
+    }
+
+
+
+    /**
+     * <p>Gets the {@link PropertySet}'s first section.</p>
+     *
+     * @return The {@link PropertySet}'s first section.
+     */
+    public Section getFirstSection()
+    {
+        if (getSectionCount() < 1)
+            throw new MissingSectionException("Property set does not contain any sections.");
+        return ((Section) sections.get(0));
     }
 
 
@@ -571,16 +608,91 @@ public class PropertySet
      * <p>If the {@link PropertySet} has only a single section this
      * method returns it.</p>
      *
-     *@return The singleSection value
-     *@throws NoSingleSectionException if the {@link PropertySet} has
-     *more or less than exactly one {@link Section}.
+     * @return The singleSection value
      */
     public Section getSingleSection()
     {
+        final int sectionCount = getSectionCount();
         if (sectionCount != 1)
-	    throw new NoSingleSectionException
-		("Property set contains " + sectionCount + " sections.");
-	return ((Section) sections.get(0));
+            throw new NoSingleSectionException
+                ("Property set contains " + sectionCount + " sections.");
+        return ((Section) sections.get(0));
     }
 
+
+
+    /**
+     * <p>Returns <code>true</code> if the <code>PropertySet</code> is equal
+     * to the specified parameter, else <code>false</code>.</p>
+     *
+     * @param o the object to compare this <code>PropertySet</code> with
+     * 
+     * @return <code>true</code> if the objects are equal, <code>false</code>
+     * if not
+     */
+    public boolean equals(final Object o)
+    {
+        if (o == null || !(o instanceof PropertySet))
+            return false;
+        final PropertySet ps = (PropertySet) o;
+        int byteOrder1 = ps.getByteOrder();
+        int byteOrder2 = getByteOrder();
+        ClassID classID1 = ps.getClassID();
+        ClassID classID2 = getClassID();
+        int format1 = ps.getFormat();
+        int format2 = getFormat();
+        int osVersion1 = ps.getOSVersion();
+        int osVersion2 = getOSVersion();
+        int sectionCount1 = ps.getSectionCount();
+        int sectionCount2 = getSectionCount();
+        if (byteOrder1 != byteOrder2      ||
+            !classID1.equals(classID2)    ||
+            format1 != format2            ||
+            osVersion1 != osVersion2      ||
+            sectionCount1 != sectionCount2)
+            return false;
+
+        /* Compare the sections: */
+        return Util.equals(getSections(), ps.getSections());
+    }
+
+
+
+    /**
+     * @see Object#hashCode()
+     */
+    public int hashCode()
+    {
+        throw new UnsupportedOperationException("FIXME: Not yet implemented.");
+    }
+
+
+
+    /**
+     * @see Object#toString()
+     */
+    public String toString()
+    {
+        final StringBuffer b = new StringBuffer();
+        final int sectionCount = getSectionCount();
+        b.append(getClass().getName());
+        b.append('[');
+        b.append("byteOrder: ");
+        b.append(getByteOrder());
+        b.append(", classID: ");
+        b.append(getClassID());
+        b.append(", format: ");
+        b.append(getFormat());
+        b.append(", OSVersion: ");
+        b.append(getOSVersion());
+        b.append(", sectionCount: ");
+        b.append(sectionCount);
+        b.append(", sections: [\n");
+        final List sections = getSections();
+        for (int i = 0; i < sectionCount; i++)
+            b.append(((Section) sections.get(i)).toString());
+        b.append(']');
+        b.append(']');
+        return b.toString();
+    }
 }
