@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -16,7 +17,6 @@
 
 package org.apache.poi.hssf.eventmodel;
 
-import java.io.IOException;
 import java.io.InputStream;
 import java.lang.reflect.Constructor;
 import java.util.ArrayList;
@@ -71,6 +71,7 @@ import org.apache.poi.hssf.record.MulBlankRecord;
 import org.apache.poi.hssf.record.MulRKRecord;
 import org.apache.poi.hssf.record.NameRecord;
 import org.apache.poi.hssf.record.NumberRecord;
+import org.apache.poi.hssf.record.PaneRecord;
 import org.apache.poi.hssf.record.PaletteRecord;
 import org.apache.poi.hssf.record.PasswordRecord;
 import org.apache.poi.hssf.record.PasswordRev4Record;
@@ -83,6 +84,7 @@ import org.apache.poi.hssf.record.ProtectionRev4Record;
 import org.apache.poi.hssf.record.RKRecord;
 import org.apache.poi.hssf.record.Record;
 import org.apache.poi.hssf.record.RecordFormatException;
+import org.apache.poi.hssf.record.RecordInputStream;
 import org.apache.poi.hssf.record.RefModeRecord;
 import org.apache.poi.hssf.record.RefreshAllRecord;
 import org.apache.poi.hssf.record.RightMarginRecord;
@@ -103,7 +105,9 @@ import org.apache.poi.hssf.record.WindowOneRecord;
 import org.apache.poi.hssf.record.WindowProtectRecord;
 import org.apache.poi.hssf.record.WindowTwoRecord;
 import org.apache.poi.hssf.record.WriteAccessRecord;
-import org.apache.poi.util.LittleEndian;
+import org.apache.poi.hssf.record.WriteProtectRecord;
+import org.apache.poi.hssf.record.FilePassRecord;
+import org.apache.poi.hssf.record.NoteRecord;
 
 
 /**
@@ -154,7 +158,9 @@ public class EventRecordFactory
                 BoolErrRecord.class, ExternSheetRecord.class, NameRecord.class,
                 LeftMarginRecord.class, RightMarginRecord.class,
                 TopMarginRecord.class, BottomMarginRecord.class,
-                PaletteRecord.class, StringRecord.class, SharedFormulaRecord.class
+                PaletteRecord.class, StringRecord.class, SharedFormulaRecord.class, 
+                WriteProtectRecord.class, FilePassRecord.class, PaneRecord.class,
+                NoteRecord.class
             };
        
     }
@@ -262,22 +268,11 @@ public class EventRecordFactory
     {
         Record    last_record = null;
 
-        try
-        {
-            short rectype = 0;
-
-            do
-            {
-                rectype = LittleEndian.readShort(in);
-                if (rectype != 0)
-                {
-                    short  recsize = LittleEndian.readShort(in);
-                    byte[] data    = new byte[ ( int ) recsize ];
-
-                    in.read(data);
-                    Record[] recs = createRecord(rectype, recsize,
-                                                 data);   // handle MulRK records
+        RecordInputStream recStream = new RecordInputStream(in);
 
+        while (recStream.hasNextRecord()) {
+          recStream.nextRecord();
+          Record[] recs = createRecord(recStream);   // handle MulRK records
                     if (recs.length > 1)
                     {
                         for (int k = 0; k < recs.length; k++)
@@ -288,8 +283,6 @@ public class EventRecordFactory
                                  break;   
                                 }
                             }
-                         //   records.add(
-                         //       recs[ k ]);               // these will be number records
                             last_record =
                                 recs[ k ];                // do to keep the algorythm homogenous...you can't
                         }                                 // actually continue a number record anyhow.
@@ -300,19 +293,6 @@ public class EventRecordFactory
 
                         if (record != null)
                         {
-                            if (rectype == ContinueRecord.sid &&
-                                ! (last_record instanceof ContinueRecord) && // include continuation records after
-                                ! (last_record instanceof UnknownRecord) )   // unknown records or previous continuation records
-                            {
-                                if (last_record == null)
-                                {
-                                    throw new RecordFormatException(
-                                        "First record is a ContinueRecord??");
-                                }
-                                last_record.processContinueRecord(data);
-                            }
-                            else
-                            {
                                 if (last_record != null) {
                                     if (throwRecordEvent(last_record) == false && abortable == true) {
                                         last_record = null;
@@ -321,35 +301,21 @@ public class EventRecordFactory
                                 }
                                 
                                 last_record = record;
-                                
-                                //records.add(record);
                             }
                         }
                     }
-                }
-            }
-            while (rectype != 0);
+
             
             if (last_record != null) {
                throwRecordEvent(last_record);               
             }
         }
-        catch (IOException e)
-        {
-            throw new RecordFormatException("Error reading bytes");
-        }
-
-        // Record[] retval = new Record[ records.size() ];
-        // retval = ( Record [] ) records.toArray(retval);
-     
-    }
 
     /**
      * create a record, if there are MUL records than multiple records
      * are returned digested into the non-mul form.
      */
-    public static Record [] createRecord(short rectype, short size,
-                                         byte [] data)
+    public static Record [] createRecord(RecordInputStream in)
     {
         Record   retval     = null;
         Record[] realretval = null;
@@ -357,25 +323,23 @@ public class EventRecordFactory
         try
         {
             Constructor constructor =
-                ( Constructor ) recordsMap.get(new Short(rectype));
+                ( Constructor ) recordsMap.get(new Short(in.getSid()));
 
             if (constructor != null)
             {
                 retval = ( Record ) constructor.newInstance(new Object[]
                 {
-                    new Short(rectype), new Short(size), data
+                    in
                 });
             }
             else
             {
-                retval = new UnknownRecord(rectype, size, data);
+                retval = new UnknownRecord(in);
             }
         }
         catch (Exception introspectionException)
         {
-            introspectionException.printStackTrace();
-            throw new RecordFormatException(
-                "Unable to construct record instance, the following exception occured: " + introspectionException.getMessage());
+            throw new RecordFormatException("Unable to construct record instance" , introspectionException);
         }
         if (retval instanceof RKRecord)
         {
@@ -470,7 +434,7 @@ public class EventRecordFactory
                 sid         = record.getField("sid").getShort(null);
                 constructor = record.getConstructor(new Class[]
                 {
-                    short.class, short.class, byte [].class
+                    RecordInputStream.class
                 });
             }
             catch (Exception illegalArgumentException)
