@@ -1,9 +1,11 @@
-/* ====================================================================
-   Copyright 2004   Apache Software Foundation
 
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+/* ====================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -13,7 +15,7 @@
    See the License for the specific language governing permissions and
    limitations under the License.
 ==================================================================== */
-
+        
 
 
 package org.apache.poi.hssf.model;
@@ -22,6 +24,7 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.regex.Pattern;
 
 //import PTG's .. since we need everything, import *
 import org.apache.poi.hssf.record.formula.*;
@@ -38,11 +41,12 @@ import org.apache.poi.hssf.record.formula.*;
  * <factor> ::= <number> | (<expression>) | <cellRef> | <function>
  * <function> ::= <functionName> ([expression [, expression]*])
  *
- *  @author Avik Sengupta <avik AT Avik Sengupta DOT com>
+ *  @author Avik Sengupta <avik at apache dot org>
  *  @author Andrew C. oliver (acoliver at apache dot org)
  *  @author Eric Ladner (eladner at goldinc dot com)
  *  @author Cameron Riley (criley at ekmail.com)
  *  @author Peter M. Murray (pete at quantrix dot com)
+ *  @author Pavel Krupets (pkrupets at palmtreebusiness dot com)
  */
 public class FormulaParser {
     
@@ -62,24 +66,32 @@ public class FormulaParser {
      * Using an unsynchronized linkedlist to implement a stack since we're not multi-threaded.
      */
     private List functionTokens = new LinkedList();
-    
-    //private Stack tokens = new java.util.Stack();
-    private List result = new ArrayList();
-    private int numParen;
-    
+
+    /**
+     * Used for spotting if we have a cell reference,
+     *  or a named range
+     */
+    private final static Pattern CELL_REFERENCE_PATTERN = Pattern.compile("(?:('?)[^:\\\\/\\?\\*\\[\\]]+\\1!)?\\$?[A-Za-z]+\\$?[\\d]+");
+        
     private static char TAB = '\t';
     private static char CR = '\n';
     
    private char look;              // Lookahead Character
-   private boolean inFunction = false;
    
    private Workbook book;
     
     
-    /** create the parser with the string that is to be parsed
-     *    later call the parse() method to return ptg list in rpn order
-     *    then call the getRPNPtg() to retrive the parse results
-     *  This class is recommended only for single threaded use
+    /** 
+     * Create the formula parser, with the string that is to be
+     *  parsed against the supplied workbook.
+     * A later call the parse() method to return ptg list in
+     *  rpn order, then call the getRPNPtg() to retrive the
+     *  parse results.
+     * This class is recommended only for single threaded use.
+     * 
+     * If you only have a usermodel.HSSFWorkbook, and not a
+     *  model.Workbook, then use the convenience method on
+     *  usermodel.HSSFFormulaEvaluator 
      */
     public FormulaParser(String formula, Workbook book){
         formulaString = formula;
@@ -114,7 +126,7 @@ public class FormulaParser {
     private void Abort(String s) {
         Error(s);
         //System.exit(1);  //throw exception??
-        throw new RuntimeException("Cannot Parse, sorry : "+s);
+        throw new RuntimeException("Cannot Parse, sorry : " + s + " @ " + pointer + " [Formula String was: '" + formulaString + "']");
     }
     
     
@@ -128,7 +140,7 @@ public class FormulaParser {
  
     /** Recognize an Alpha Character */
     private boolean IsAlpha(char c) {
-        return Character.isLetter(c) || c == '$';
+        return Character.isLetter(c) || c == '$' || c=='_';
     }
     
     
@@ -235,16 +247,19 @@ public class FormulaParser {
     
     /** Get a Number */
     private String GetNum() {
-        String Value ="";
-        if  (!IsDigit(look)) Expected("Integer");
-        while (IsDigit(look)){
-            Value = Value + look;
+        StringBuffer value = new StringBuffer();
+        
+        while (IsDigit(this.look)){
+            value.append(this.look);
             GetChar();
         }
+        
         SkipWhite();
-        return Value;
+        
+        return value.length() == 0 ? null : value.toString();
     }
-
+        
+    
     /** Output a String with Tab */
     private void  Emit(String s){
         System.out.print(TAB+s);
@@ -263,9 +278,14 @@ public class FormulaParser {
         if (look == '('){
             //This is a function
             function(name);
-        } else if (look == ':') { // this is a AreaReference
+        } else if (look == ':' || look == '.') { // this is a AreaReference
+            GetChar();
+            
+            while (look == '.') { // formulas can have . or .. or ... instead of :
+                GetChar();
+            }
+            
             String first = name;
-            Match(':');
             String second = GetName();
             tokens.add(new AreaPtg(first+":"+second));
         } else if (look == '!') {
@@ -276,21 +296,44 @@ public class FormulaParser {
             if (look == ':') {
                 Match(':');
                 String second=GetName();
-                
-                tokens.add(new Area3DPtg(first+":"+second,externIdx));
+                if (look == '!') {
+                	//The sheet name was included in both of the areas. Only really
+                	//need it once
+                	Match('!');
+                	String third=GetName();
+                	
+                	if (!sheetName.equals(second))
+                		throw new RuntimeException("Unhandled double sheet reference.");
+                	
+                	tokens.add(new Area3DPtg(first+":"+third,externIdx));
+                } else {                                  
+                  tokens.add(new Area3DPtg(first+":"+second,externIdx));
+                }
             } else {
                 tokens.add(new Ref3DPtg(first,externIdx));
             }
         } else {
-            //this can be either a cell ref or a named range !!
-            boolean cellRef = true ; //we should probably do it with reg exp??
+            // This can be either a cell ref or a named range
+        	// Try to spot which it is
+        	boolean cellRef = CELL_REFERENCE_PATTERN.matcher(name).matches();
             boolean boolLit = (name.equals("TRUE") || name.equals("FALSE"));
+            
             if (boolLit) {
                 tokens.add(new BoolPtg(name));
             } else if (cellRef) {
                 tokens.add(new ReferencePtg(name));
-            }else {
-                //handle after named range is integrated!!
+            } else {
+            	boolean nameRecordExists = false;
+                for(int i = 0; i < book.getNumNames(); i++) {
+                	// Our formula will by now contain an upper-cased
+                	//  version of any named range names
+                    if(book.getNameRecord(i).getNameText().toUpperCase().equals(name)) {
+                        nameRecordExists = true;
+                    }
+                }
+                if(!nameRecordExists)
+                    Abort("Found reference to named range \"" + name + "\", but that named range wasn't defined!");
+                tokens.add(new NamePtg(name, book));
             }
         }
     }
@@ -465,6 +508,11 @@ public class FormulaParser {
     		Factor();
     		tokens.add(new UnaryMinusPtg());
     	}
+        else if (look == '+') {
+            Match('+');
+            Factor();
+            tokens.add(new UnaryPlusPtg());
+        }
         else if (look == '(' ) {
             Match('(');
             Expression();
@@ -474,21 +522,85 @@ public class FormulaParser {
             Ident();
         } else if(look == '"') {
            StringLiteral();
+        } else if (look == ')' || look == ',') {
+        	tokens.add(new MissingArgPtg());
         } else {
-             
-            String number = GetNum();
-            if (look=='.') {
-                Match('.');
-                String decimalPart = null;
-                if (IsDigit(look)) number = number +"."+ GetNum(); //this also takes care of someone entering "1234."
-                tokens.add(new NumberPtg(number));
-            } else {
-                tokens.add(new IntPtg(number));  //TODO:what if the number is too big to be a short? ..add factory to return Int or Number!
+            String number2 = null;
+            String exponent = null;
+            String number1 = GetNum();
+            
+            if (look == '.') {
+                GetChar();
+                number2 = GetNum();
             }
+            
+            if (look == 'E') {
+                GetChar();
+                
+                String sign = "";
+                if (look == '+') {
+                    GetChar();
+                } else if (look == '-') {
+                    GetChar();
+                    sign = "-";
+                }
+                
+                String number = GetNum();
+                if (number == null) {
+                    Expected("Integer");
+                }
+                exponent = sign + number;
+            }
+            
+            if (number1 == null && number2 == null) {
+                Expected("Integer");
+            }
+            
+            tokens.add(getNumberPtgFromString(number1, number2, exponent));
         }
     }
     
-    private void StringLiteral() 
+	/** 
+	 * Get a PTG for an integer from its string representation. 
+	 * return Int or Number Ptg based on size of input
+	 */
+	private Ptg getNumberPtgFromString(String number1, String number2, String exponent) {
+        StringBuffer number = new StringBuffer();
+        
+	    if (number2 == null) {
+	        number.append(number1);
+    	    
+    	    if (exponent != null) {
+    	        number.append('E');
+    	        number.append(exponent);
+    	    }
+    	    
+            String numberStr = number.toString();
+            
+            try {
+                return new IntPtg(numberStr);
+            } catch (NumberFormatException e) {
+                return new NumberPtg(numberStr);
+            }
+	    } else {
+            if (number1 != null) {
+                number.append(number1);
+            }
+            
+            number.append('.');
+            number.append(number2);
+            
+            if (exponent != null) {
+                number.append('E');
+                number.append(exponent);
+            }
+            
+            return new NumberPtg(number.toString());
+	    }
+	}
+	
+	
+	private void StringLiteral() 
 	{
 		// Can't use match here 'cuz it consumes whitespace
 		// which we need to preserve inside the string.
@@ -796,8 +908,8 @@ end;
         }
      }
     /**
-     * Convience method which takes in a list then passes it to the other toFormulaString
-     * signature. 
+     * Convience method which takes in a list then passes it to the
+     *  other toFormulaString signature. 
      * @param book   workbook for 3D and named references
      * @param lptgs  list of Ptg, can be null or empty
      * @return a human readable String
@@ -810,6 +922,16 @@ end;
         retval = toFormulaString(book, ptgs);
         return retval;
     }
+    /**
+     * Convience method which takes in a list then passes it to the
+     *  other toFormulaString signature. Works on the current
+     *  workbook for 3D and named references
+     * @param lptgs  list of Ptg, can be null or empty
+     * @return a human readable String
+     */
+    public String toFormulaString(List lptgs) {
+    	return toFormulaString(book, lptgs);
+    }
     
     /**
      * Static method to convert an array of Ptgs in RPN order
@@ -875,6 +997,16 @@ end;
         // TODO: catch stack underflow and throw parse exception.
         return (String) stack.pop();
     }
+    /**
+     * Static method to convert an array of Ptgs in RPN order
+     *  to a human readable string format in infix mode. Works
+     *  on the current workbook for named and 3D references.
+     * @param ptgs  array of Ptg, can be null or empty
+     * @return a human readable String
+     */
+    public String toFormulaString(Ptg[] ptgs) {
+    	return toFormulaString(book, ptgs);
+    }
 
 
     /** Create a tree representation of the RPN token array
