@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -17,13 +18,12 @@
 
 package org.apache.poi.hssf.model;
 
+import org.apache.poi.ddf.*;
 import org.apache.poi.hssf.record.*;
 import org.apache.poi.hssf.util.HSSFColor;
 import org.apache.poi.hssf.util.SheetReferences;
-import org.apache.poi.hssf.usermodel.HSSFSheet;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
-import org.apache.poi.ddf.*;
 
 import java.util.ArrayList;
 import java.util.Iterator;
@@ -44,13 +44,13 @@ import java.util.Locale;
  * before even attempting to use this.
  *
  *
+ * @author  Luc Girardin (luc dot girardin at macrofocus dot com)
+ * @author  Sergei Kozello (sergeikozello at mail.ru)
  * @author  Shawn Laubach (slaubach at apache dot org) (Data Formats)
  * @author  Andrew C. Oliver (acoliver at apache dot org)
- * @author  Glen Stampoultzis (glens at apache.org)
- * @author  Sergei Kozello (sergeikozello at mail.ru)
- * @author  Luc Girardin (luc dot girardin at macrofocus dot com)
- * @author  Dan Sherman (dsherman at isisph.com)
  * @author  Brian Sanders (bsanders at risklabs dot com) - custom palette
+ * @author  Dan Sherman (dsherman at isisph.com)
+ * @author  Glen Stampoultzis (glens at apache.org)
  * @see org.apache.poi.hssf.usermodel.HSSFWorkbook
  * @version 1.0-pre
  */
@@ -98,7 +98,12 @@ public class Workbook implements Model
     protected int              numfonts    = 0;   // hold the number of font records
     private short              maxformatid  = -1;  // holds the max format id
     private boolean            uses1904datewindowing  = false;  // whether 1904 date windowing is being used
-    private DrawingManager drawingManager;
+    private DrawingManager2    drawingManager;
+    private List               escherBSERecords = new ArrayList();  // EscherBSERecord
+    private WindowOneRecord windowOne;
+    private FileSharingRecord fileShare;
+    private WriteAccessRecord writeAccess;
+    private WriteProtectRecord writeProtect;
 
     private static POILogger   log = POILogFactory.getLogger(Workbook.class);
 
@@ -214,6 +219,26 @@ public class Workbook implements Model
                     if (log.check( POILogger.DEBUG ))
                         log.log(DEBUG, "found palette record at " + k);
                     retval.records.setPalettepos( k );
+                    break;
+                case WindowOneRecord.sid:
+                    if (log.check( POILogger.DEBUG ))
+                        log.log(DEBUG, "found WindowOneRecord at " + k);
+                    retval.windowOne = (WindowOneRecord) rec; 
+                    break;
+                case WriteAccessRecord.sid: 
+                    if (log.check( POILogger.DEBUG ))
+                        log.log(DEBUG, "found WriteAccess at " + k);
+                    retval.writeAccess = (WriteAccessRecord) rec;
+                    break;
+                case WriteProtectRecord.sid: 
+                    if (log.check( POILogger.DEBUG ))
+                        log.log(DEBUG, "found WriteProtect at " + k);
+                    retval.writeProtect = (WriteProtectRecord) rec;
+                    break;
+                case FileSharingRecord.sid: 
+                    if (log.check( POILogger.DEBUG ))
+                        log.log(DEBUG, "found FileSharing at " + k);
+                    retval.fileShare = (FileSharingRecord) rec;
                 default :
             }
             records.add(rec);
@@ -225,6 +250,10 @@ public class Workbook implements Model
         //        }
 
         retval.records.setRecords(records);
+        
+        if (retval.windowOne == null) {
+            retval.windowOne = (WindowOneRecord) retval.createWindowOne();
+        }
         if (log.check( POILogger.DEBUG ))
             log.log(DEBUG, "exit create workbook from existing file function");
         return retval;
@@ -258,7 +287,8 @@ public class Workbook implements Model
         records.add( retval.createPassword() );
         records.add( retval.createProtectionRev4() );
         records.add( retval.createPasswordRev4() );
-        records.add( retval.createWindowOne() );
+        retval.windowOne = (WindowOneRecord) retval.createWindowOne();
+        records.add( retval.windowOne );
         records.add( retval.createBackup() );
         retval.records.setBackuppos( records.size() - 1 );
         records.add( retval.createHideObj() );
@@ -446,12 +476,39 @@ public class Workbook implements Model
      * @param sheetnum the sheet number (0 based)
      * @param sheetname the name for the sheet
      */
-
-    // for compatibility
     public void setSheetName(int sheetnum, String sheetname ) {
-        setSheetName( sheetnum, sheetname, (byte)0 );
+        checkSheets(sheetnum);
+        BoundSheetRecord sheet = (BoundSheetRecord)boundsheets.get( sheetnum );
+        sheet.setSheetname(sheetname);
+        sheet.setSheetnameLength( (byte)sheetname.length() );
     }
 
+    /**
+     * Determines whether a workbook contains the privided sheet name.
+     *
+     * @param name the name to test
+     * @param excludeSheetIdx the sheet to exclude from the check or -1 to include all sheets in the check.
+     * @return true if the sheet contains the name, false otherwise.
+     */
+    public boolean doesContainsSheetName( String name, int excludeSheetIdx )
+    {
+        for ( int i = 0; i < boundsheets.size(); i++ )
+        {
+            BoundSheetRecord boundSheetRecord = (BoundSheetRecord) boundsheets.get( i );
+            if (excludeSheetIdx != i && name.equals(boundSheetRecord.getSheetname()))
+                return true;
+        }
+        return false;
+    }
+
+    /**
+     * sets the name for a given sheet forcing the encoding. This is STILL A BAD IDEA.
+     * Poi now automatically detects unicode
+     *
+     *@deprecated 3-Jan-06 Simply use setSheetNam e(int sheetnum, String sheetname)
+     * @param sheetnum the sheet number (0 based)
+     * @param sheetname the name for the sheet
+     */    
     public void setSheetName(int sheetnum, String sheetname, short encoding ) {
         checkSheets(sheetnum);
         BoundSheetRecord sheet = (BoundSheetRecord)boundsheets.get( sheetnum );
@@ -460,7 +517,7 @@ public class Workbook implements Model
 		sheet.setCompressedUnicodeFlag( (byte)encoding );
     }
     
-    	/**
+    /**
 	 * sets the order of appearance for a given sheet.
 	 *
 	 * @param sheetname the name of the sheet to reorder
@@ -485,6 +542,29 @@ public class Workbook implements Model
         .getSheetname();
     }
 
+    /**
+     * gets the hidden flag for a given sheet.
+     *
+     * @param sheetnum the sheet number (0 based)
+     * @return True if sheet is hidden
+     */
+
+    public boolean isSheetHidden(int sheetnum) {
+        BoundSheetRecord bsr = ( BoundSheetRecord ) boundsheets.get(sheetnum);
+        return bsr.isHidden();
+    }
+
+    /**
+     * Hide or unhide a sheet
+     * 
+     * @param sheetnum The sheet number
+     * @param hidden True to mark the sheet as hidden, false otherwise
+     */
+    
+    public void setSheetHidden(int sheetnum, boolean hidden) {
+        BoundSheetRecord bsr = ( BoundSheetRecord ) boundsheets.get(sheetnum);
+        bsr.setHidden(hidden);
+    }
     /**
      * get the sheet's index
      * @param name  sheet name
@@ -604,25 +684,6 @@ public class Workbook implements Model
         return xf;
     }
 
-    /**
-     * Adds a string to the SST table and returns its index (if its a duplicate
-     * just returns its index and update the counts)
-     *
-     * @param string the string to be added to the SSTRecord
-     * @param use16bits whether to use utf 16 or false for compressed unicode
-     * @return index of the string within the SSTRecord
-     */
-
-    public int addSSTString(String string, boolean use16bits) {
-        if (log.check( POILogger.DEBUG ))
-            log.log(DEBUG, "insert to sst string='", string, "' and use16bits= ",
-        new Boolean(use16bits));
-        if (sst == null) {
-            insertSST();
-        }
-        return sst.addString(string, use16bits);
-    }
-
     /**
      * Adds a string to the SST table and returns its index (if its a duplicate
      * just returns its index and update the counts) ASSUMES compressed unicode
@@ -633,8 +694,13 @@ public class Workbook implements Model
      * @return index of the string within the SSTRecord
      */
 
-    public int addSSTString(String string) {
-        return addSSTString(string, false);
+    public int addSSTString(UnicodeString string) {
+        if (log.check( POILogger.DEBUG ))
+          log.log(DEBUG, "insert to sst string='", string);
+        if (sst == null) {
+            insertSST();
+        }
+      return sst.addString(string);
     }
 
     /**
@@ -642,11 +708,11 @@ public class Workbook implements Model
      * @return String containing the SST String
      */
 
-    public String getSSTString(int str) {
+    public UnicodeString getSSTString(int str) {
         if (sst == null) {
             insertSST();
         }
-        String retval = sst.getString(str);
+        UnicodeString retval = sst.getString(str);
 
         if (log.check( POILogger.DEBUG ))
             log.log(DEBUG, "Returning SST for index=", new Integer(str),
@@ -713,6 +779,7 @@ public class Workbook implements Model
 
         SSTRecord sst = null;
         int sstPos = 0;
+        boolean wroteBoundSheets = false;
         for ( int k = 0; k < records.size(); k++ )
         {
 
@@ -720,6 +787,7 @@ public class Workbook implements Model
             // Let's skip RECALCID records, as they are only use for optimization
             if ( record.getSid() != RecalcIdRecord.sid || ( (RecalcIdRecord) record ).isNeeded() )
             {
+                int len = 0; 
                 if (record instanceof SSTRecord)
                 {
                     sst = (SSTRecord)record;
@@ -729,7 +797,22 @@ public class Workbook implements Model
                 {
                     record = sst.createExtSSTRecord(sstPos + offset);
                 }
-                pos += record.serialize( pos + offset, data );   // rec.length;
+                if (record instanceof BoundSheetRecord) {
+                     if(!wroteBoundSheets) {
+                        for (int i = 0; i < boundsheets.size(); i++) {
+                            len+= ((BoundSheetRecord)boundsheets.get(i))
+                                             .serialize(pos+offset+len, data);
+                        }
+                        wroteBoundSheets = true;
+                     }
+                } else {
+                   len = record.serialize( pos + offset, data );
+                }
+                /////  DEBUG BEGIN /////
+//                if (len != record.getRecordSize())
+//                    throw new IllegalStateException("Record size does not match serialized bytes.  Serialized size = " + len + " but getRecordSize() returns " + record.getRecordSize());
+                /////  DEBUG END /////
+                pos += len;   // rec.length;
             }
         }
         if (log.check( POILogger.DEBUG ))
@@ -830,7 +913,16 @@ public class Workbook implements Model
     protected Record createWriteAccess() {
         WriteAccessRecord retval = new WriteAccessRecord();
 
-        retval.setUsername(System.getProperty("user.name"));
+        try
+        {
+            retval.setUsername(System.getProperty("user.name"));
+        }
+        catch (java.security.AccessControlException e)
+        {
+                // AccessControlException can occur in a restricted context
+                // (client applet/jws application or restricted security server)
+                retval.setUsername("POI");
+        }
         return retval;
     }
 
@@ -1598,7 +1690,7 @@ public class Workbook implements Model
      */
     protected PaletteRecord createPalette()
     {
-        return new PaletteRecord(PaletteRecord.sid);
+        return new PaletteRecord();
     }
     
     /**
@@ -1974,6 +2066,7 @@ public class Workbook implements Model
         return maxformatid;
     }
 
+  
 
     /**
      * Returns the first occurance of a record matching a particular sid.
@@ -2072,15 +2165,69 @@ public class Workbook implements Model
       }
       return palette;
     }
+    
+    /**
+     * Finds the primary drawing group, if one already exists
+     */
+    public void findDrawingGroup() {
+    	// Need to find a DrawingGroupRecord that
+    	//  contains a EscherDggRecord
+    	for(Iterator rit = records.iterator(); rit.hasNext();) {
+    		Record r = (Record)rit.next();
+    		
+    		if(r instanceof DrawingGroupRecord) {
+            	DrawingGroupRecord dg =	(DrawingGroupRecord)r;
+            	dg.processChildRecords();
+            	
+            	EscherContainerRecord cr =
+            		dg.getEscherContainer();
+            	if(cr == null) {
+            		continue;
+            	}
+            	
+            	EscherDggRecord dgg = null;
+            	for(Iterator it = cr.getChildRecords().iterator(); it.hasNext();) {
+            		Object er = it.next();
+            		if(er instanceof EscherDggRecord) {
+            			dgg = (EscherDggRecord)er;
+            		}
+            	}
+            	
+            	if(dgg != null) {
+            		drawingManager = new DrawingManager2(dgg);
+            		return;
+            	}
+    		}
+    	}
+
+    	// Look for the DrawingGroup record
+        int dgLoc = findFirstRecordLocBySid(DrawingGroupRecord.sid);
+        
+    	// If there is one, does it have a EscherDggRecord?
+        if(dgLoc != -1) {
+        	DrawingGroupRecord dg =
+        		(DrawingGroupRecord)records.get(dgLoc);
+        	EscherDggRecord dgg = null;
+        	for(Iterator it = dg.getEscherRecords().iterator(); it.hasNext();) {
+        		Object er = it.next();
+        		if(er instanceof EscherDggRecord) {
+        			dgg = (EscherDggRecord)er;
+        		}
+        	}
+        	
+        	if(dgg != null) {
+        		drawingManager = new DrawingManager2(dgg);
+        	}
+        }
+    }
 
     /**
-     * Creates a drawing group record.  If it already exists then it's left
-     * alone.
+     * Creates a primary drawing group record.  If it already 
+     *  exists then it's modified.
      */
     public void createDrawingGroup()
     {
-        int dggLoc = findFirstRecordLocBySid(EscherContainerRecord.DGG_CONTAINER);
-        if (dggLoc == -1)
+        if (drawingManager == null)
         {
             EscherContainerRecord dggContainer = new EscherContainerRecord();
             EscherDggRecord dgg = new EscherDggRecord();
@@ -2095,11 +2242,23 @@ public class Workbook implements Model
             dgg.setNumShapesSaved(0);
             dgg.setDrawingsSaved(0);
             dgg.setFileIdClusters(new EscherDggRecord.FileIdCluster[] {} );
-            drawingManager = new DrawingManager(dgg);
+            drawingManager = new DrawingManager2(dgg);
+            EscherContainerRecord bstoreContainer = null;
+            if (escherBSERecords.size() > 0)
+            {
+                bstoreContainer = new EscherContainerRecord();
+                bstoreContainer.setRecordId( EscherContainerRecord.BSTORE_CONTAINER );
+                bstoreContainer.setOptions( (short) ( (escherBSERecords.size() << 4) | 0xF ) );
+                for ( Iterator iterator = escherBSERecords.iterator(); iterator.hasNext(); )
+                {
+                    EscherRecord escherRecord = (EscherRecord) iterator.next();
+                    bstoreContainer.addChildRecord( escherRecord );
+                }
+            }
             opt.setRecordId((short) 0xF00B);
             opt.setOptions((short) 0x0033);
             opt.addEscherProperty( new EscherBoolProperty(EscherProperties.TEXT__SIZE_TEXT_TO_FIT_SHAPE, 524296) );
-            opt.addEscherProperty( new EscherRGBProperty(EscherProperties.FILL__FILLCOLOR, 134217737) );
+            opt.addEscherProperty( new EscherRGBProperty(EscherProperties.FILL__FILLCOLOR, 0x08000041) );
             opt.addEscherProperty( new EscherRGBProperty(EscherProperties.LINESTYLE__COLOR, 134217792) );
             splitMenuColors.setRecordId((short) 0xF11E);
             splitMenuColors.setOptions((short) 0x0040);
@@ -2109,20 +2268,137 @@ public class Workbook implements Model
             splitMenuColors.setColor4(0x100000F7);
 
             dggContainer.addChildRecord(dgg);
+            if (bstoreContainer != null)
+                dggContainer.addChildRecord( bstoreContainer );
             dggContainer.addChildRecord(opt);
             dggContainer.addChildRecord(splitMenuColors);
 
-            DrawingGroupRecord drawingGroup = new DrawingGroupRecord();
-            drawingGroup.addEscherRecord(dggContainer);
-            int loc = findFirstRecordLocBySid(CountryRecord.sid);
-            getRecords().add(loc+1, drawingGroup);
+            int dgLoc = findFirstRecordLocBySid(DrawingGroupRecord.sid);
+            if (dgLoc == -1)
+            {
+                DrawingGroupRecord drawingGroup = new DrawingGroupRecord();
+                drawingGroup.addEscherRecord(dggContainer);
+                int loc = findFirstRecordLocBySid(CountryRecord.sid);
+
+                getRecords().add(loc+1, drawingGroup);
+            }
+            else
+            {
+                DrawingGroupRecord drawingGroup = new DrawingGroupRecord();
+                drawingGroup.addEscherRecord(dggContainer);
+                getRecords().set(dgLoc, drawingGroup);
+            }
+
         }
     }
+    
+    public WindowOneRecord getWindowOne() {
+        return windowOne;
+    }
 
-    public DrawingManager getDrawingManager()
+    public EscherBSERecord getBSERecord(int pictureIndex)
+    {
+        return (EscherBSERecord)escherBSERecords.get(pictureIndex-1);
+    }
+
+    public int addBSERecord(EscherBSERecord e)
+    {
+        createDrawingGroup();
+
+        // maybe we don't need that as an instance variable anymore
+        escherBSERecords.add( e );
+
+        int dgLoc = findFirstRecordLocBySid(DrawingGroupRecord.sid);
+        DrawingGroupRecord drawingGroup = (DrawingGroupRecord) getRecords().get( dgLoc );
+
+        EscherContainerRecord dggContainer = (EscherContainerRecord) drawingGroup.getEscherRecord( 0 );
+        EscherContainerRecord bstoreContainer;
+        if (dggContainer.getChild( 1 ).getRecordId() == EscherContainerRecord.BSTORE_CONTAINER )
+        {
+            bstoreContainer = (EscherContainerRecord) dggContainer.getChild( 1 );
+        }
+        else
+        {
+            bstoreContainer = new EscherContainerRecord();
+            bstoreContainer.setRecordId( EscherContainerRecord.BSTORE_CONTAINER );
+            dggContainer.getChildRecords().add( 1, bstoreContainer );
+        }
+        bstoreContainer.setOptions( (short) ( (escherBSERecords.size() << 4) | 0xF ) );
+
+        bstoreContainer.addChildRecord( e );
+
+        return escherBSERecords.size();
+    }
+
+    public DrawingManager2 getDrawingManager()
     {
         return drawingManager;
     }
 
+    public WriteProtectRecord getWriteProtect() {
+        if (this.writeProtect == null) {
+           this.writeProtect = new WriteProtectRecord();
+           int i = 0;
+           for (i = 0; 
+                i < records.size() && !(records.get(i) instanceof BOFRecord); 
+                i++) {
+           }
+           records.add(i+1,this.writeProtect);
+        }
+        return this.writeProtect;
+    }
+
+    public WriteAccessRecord getWriteAccess() {
+        if (this.writeAccess == null) {
+           this.writeAccess = (WriteAccessRecord)createWriteAccess();
+           int i = 0;
+           for (i = 0; 
+                i < records.size() && !(records.get(i) instanceof InterfaceEndRecord); 
+                i++) {
+           }
+           records.add(i+1,this.writeAccess);
+        }
+        return this.writeAccess;
+    }
+
+    public FileSharingRecord getFileSharing() {
+        if (this.fileShare == null) {
+           this.fileShare = new FileSharingRecord();
+           int i = 0;
+           for (i = 0; 
+                i < records.size() && !(records.get(i) instanceof WriteAccessRecord); 
+                i++) {
+           }
+           records.add(i+1,this.fileShare);
+        }
+        return this.fileShare;
+    }
+
+    /**
+     * protect a workbook with a password (not encypted, just sets writeprotect
+     * flags and the password.
+     * @param password to set
+     */
+    public void writeProtectWorkbook( String password, String username ) {
+        int protIdx = -1;
+        FileSharingRecord frec = getFileSharing();
+        WriteAccessRecord waccess = getWriteAccess();
+        WriteProtectRecord wprotect = getWriteProtect();
+        frec.setReadOnly((short)1);
+        frec.setPassword(FileSharingRecord.hashPassword(password));
+        frec.setUsername(username);
+        waccess.setUsername(username);
+    }
+
+    /**
+     * removes the write protect flag
+     */
+    public void unwriteProtectWorkbook() {
+        records.remove(fileShare);
+        records.remove(writeProtect);
+        fileShare = null;
+        writeProtect = null;
+    }
+
 }
 
