@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2003-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -24,14 +25,9 @@ public class DrawingSelectionRecord extends AbstractEscherHolderRecord
     {
     }
 
-    public DrawingSelectionRecord( short id, short size, byte[] data )
+    public DrawingSelectionRecord( RecordInputStream in )
     {
-        super( id, size, data );
-    }
-
-    public DrawingSelectionRecord( short id, short size, byte[] data, int offset )
-    {
-        super( id, size, data, offset );
+        super( in );
     }
 
     protected String getRecordName()
