@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2003-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -20,8 +21,11 @@ import org.apache.poi.ddf.*;
 import org.apache.poi.hssf.usermodel.*;
 import org.apache.poi.hssf.model.AbstractShape;
 import org.apache.poi.hssf.model.TextboxShape;
-import org.apache.poi.hssf.model.DrawingManager;
+import org.apache.poi.hssf.model.DrawingManager2;
 import org.apache.poi.hssf.model.ConvertAnchor;
+import org.apache.poi.hssf.model.CommentShape;
+import org.apache.poi.util.POILogFactory;
+import org.apache.poi.util.POILogger;
 
 import java.util.*;
 
@@ -45,6 +49,7 @@ import java.util.*;
 public class EscherAggregate extends AbstractEscherHolderRecord
 {
     public static final short sid = 9876;
+    private static POILogger log = POILogFactory.getLogger(EscherAggregate.class);
 
     public static final short ST_MIN = (short) 0;
     public static final short ST_NOT_PRIMATIVE = ST_MIN;
@@ -256,10 +261,15 @@ public class EscherAggregate extends AbstractEscherHolderRecord
 
     /** Maps shape container objects to their OBJ records */
     private Map shapeToObj = new HashMap();
-    private DrawingManager drawingManager;
+    private DrawingManager2 drawingManager;
     private short drawingGroupId;
 
-    public EscherAggregate( DrawingManager drawingManager )
+    /**
+     * list of "tail" records that need to be serialized after all drawing group records
+     */
+    private List tailRec = new ArrayList();
+
+    public EscherAggregate( DrawingManager2 drawingManager )
     {
         this.drawingManager = drawingManager;
     }
@@ -305,7 +315,7 @@ public class EscherAggregate extends AbstractEscherHolderRecord
     /**
      * Collapses the drawing records into an aggregate.
      */
-    public static EscherAggregate createAggregate( List records, int locFirstDrawingRecord, DrawingManager drawingManager )
+    public static EscherAggregate createAggregate( List records, int locFirstDrawingRecord, DrawingManager2 drawingManager )
     {
         // Keep track of any shape records created so we can match them back to the object id's.
         // Textbox objects are also treated as shape objects.
@@ -449,6 +459,13 @@ public class EscherAggregate extends AbstractEscherHolderRecord
 
         }
 
+        // write records that need to be serialized after all drawing group records
+        for ( int i = 0; i < tailRec.size(); i++ )
+        {
+            Record rec = (Record)tailRec.get(i);
+            pos += rec.serialize( pos, data );
+        }
+
         int bytesWritten = pos - offset;
         if ( bytesWritten != getRecordSize() )
             throw new RecordFormatException( bytesWritten + " bytes written but getRecordSize() reports " + getRecordSize() );
@@ -483,7 +500,13 @@ public class EscherAggregate extends AbstractEscherHolderRecord
             Record r = (Record) iterator.next();
             objRecordSize += r.getRecordSize();
         }
-        return drawingRecordSize + objRecordSize;
+        int tailRecordSize = 0;
+        for ( Iterator iterator = tailRec.iterator(); iterator.hasNext(); )
+        {
+            Record r = (Record) iterator.next();
+            tailRecordSize += r.getRecordSize();
+        }
+        return drawingRecordSize + objRecordSize + tailRecordSize;
     }
 
     /**
@@ -503,7 +526,146 @@ public class EscherAggregate extends AbstractEscherHolderRecord
     {
         this.patriarch = patriarch;
     }
+    
+    /**
+     * Converts the Records into UserModel
+     *  objects on the bound HSSFPatriarch
+     */
+    public void convertRecordsToUserModel() {
+    	if(patriarch == null) {
+    		throw new IllegalStateException("Must call setPatriarch() first");
+    	}
+    	
+    	// The top level container ought to have
+    	//  the DgRecord and the container of one container
+    	//  per shape group (patriach overall first)
+    	EscherContainerRecord topContainer =
+    		(EscherContainerRecord)getEscherContainer();
+    	if(topContainer == null) {
+    		return;
+    	}
+    	topContainer = (EscherContainerRecord) 
+    		topContainer.getChildContainers().get(0);
+    	
+    	List tcc = topContainer.getChildContainers();
+    	if(tcc.size() == 0) {
+    		throw new IllegalStateException("No child escher containers at the point that should hold the patriach data, and one container per top level shape!");
+    	}
+    	
+    	// First up, get the patriach position
+    	// This is in the first EscherSpgrRecord, in
+    	//  the first container, with a EscherSRecord too
+    	EscherContainerRecord patriachContainer =
+    		(EscherContainerRecord)tcc.get(0);
+    	EscherSpgrRecord spgr = null;
+    	for(Iterator it = patriachContainer.getChildRecords().iterator(); it.hasNext();) {
+    		EscherRecord r = (EscherRecord)it.next();
+    		if(r instanceof EscherSpgrRecord) {
+    			spgr = (EscherSpgrRecord)r;
+    			break;
+    		}
+    	}
+    	if(spgr != null) {
+    		patriarch.setCoordinates(
+    				spgr.getRectX1(), spgr.getRectY1(),
+    				spgr.getRectX2(), spgr.getRectY2()
+    		);
+    	}
+    	
+    	// Now process the containers for each group
+    	//  and objects
+    	for(int i=1; i<tcc.size(); i++) {
+    		EscherContainerRecord shapeContainer =
+    			(EscherContainerRecord)tcc.get(i);
+    		//System.err.println("\n\n*****\n\n");
+    		//System.err.println(shapeContainer);
+    		
+    		// Could be a group, or a base object
+    		if(shapeContainer.getChildRecords().size() == 1 &&
+    				shapeContainer.getChildContainers().size() == 1) {
+    			// Group
+    			HSSFShapeGroup group =
+    				new HSSFShapeGroup(null, new HSSFClientAnchor());
+    			patriarch.getChildren().add(group);
+    			
+    			EscherContainerRecord groupContainer =
+    				(EscherContainerRecord)shapeContainer.getChild(0);
+    			convertRecordsToUserModel(groupContainer, group);
+    		} else if(shapeContainer.hasChildOfType((short)0xF00D)) {
+    			// TextBox
+    			HSSFTextbox box =
+    				new HSSFTextbox(null, new HSSFClientAnchor());
+    			patriarch.getChildren().add(box);
+    			
+    			convertRecordsToUserModel(shapeContainer, box);
+    		} else if(shapeContainer.hasChildOfType((short)0xF011)) {
+    			// Not yet supporting EscherClientDataRecord stuff
+    		} else {
+    			// Base level
+    			convertRecordsToUserModel(shapeContainer, patriarch);
+    		}
+    	}
+    	
+    	// Now, clear any trace of what records make up
+    	//  the patriarch
+    	// Otherwise, everything will go horribly wrong
+    	//  when we try to write out again....
+//    	clearEscherRecords();
+    	drawingManager.getDgg().setFileIdClusters(new EscherDggRecord.FileIdCluster[0]);
 
+    	// TODO: Support converting our records
+    	//  back into shapes
+    	log.log(POILogger.WARN, "Not processing objects into Patriarch!");
+    }
+    
+    private void convertRecordsToUserModel(EscherContainerRecord shapeContainer, Object model) {
+    	for(Iterator it = shapeContainer.getChildRecords().iterator(); it.hasNext();) {
+    		EscherRecord r = (EscherRecord)it.next();
+    		if(r instanceof EscherSpgrRecord) {
+    			// This may be overriden by a later EscherClientAnchorRecord
+    			EscherSpgrRecord spgr = (EscherSpgrRecord)r;
+    			
+    			if(model instanceof HSSFShapeGroup) {
+    				HSSFShapeGroup g = (HSSFShapeGroup)model;
+    				g.setCoordinates(
+    						spgr.getRectX1(), spgr.getRectY1(),
+    						spgr.getRectX2(), spgr.getRectY2()
+    				);
+    			} else {
+    				throw new IllegalStateException("Got top level anchor but not processing a group");
+    			}
+    		} 
+    		else if(r instanceof EscherClientAnchorRecord) {
+    			EscherClientAnchorRecord car = (EscherClientAnchorRecord)r;
+    			
+    			if(model instanceof HSSFShape) {
+    				HSSFShape g = (HSSFShape)model;
+    				g.getAnchor().setDx1(car.getDx1());
+    				g.getAnchor().setDx2(car.getDx2());
+    				g.getAnchor().setDy1(car.getDy1());
+    				g.getAnchor().setDy2(car.getDy2());
+    			} else {
+    				throw new IllegalStateException("Got top level anchor but not processing a group or shape");
+    			}
+    		}
+    		else if(r instanceof EscherTextboxRecord) {
+    			EscherTextboxRecord tbr = (EscherTextboxRecord)r;
+    			
+    			// Also need to find the TextObjectRecord too
+    			// TODO
+    		}
+    		else if(r instanceof EscherSpRecord) {
+    			// Use flags if needed
+    		}
+    		else if(r instanceof EscherOptRecord) {
+    			// Use properties if needed
+    		}
+    		else {
+    			//System.err.println(r);
+    		}
+    	}
+    }
+    
     public void clear()
     {
         clearEscherRecords();
@@ -528,6 +690,7 @@ public class EscherAggregate extends AbstractEscherHolderRecord
         if ( patriarch != null )
         {
             shapeToObj.clear();
+            tailRec.clear();
             clearEscherRecords();
             if ( patriarch.getChildren().size() != 0 )
             {
@@ -567,10 +730,19 @@ public class EscherAggregate extends AbstractEscherHolderRecord
                     EscherRecord escherTextbox = ( (TextboxShape) shapeModel ).getEscherTextbox();
                     shapeToObj.put( escherTextbox, ( (TextboxShape) shapeModel ).getTextObjectRecord() );
                     //                    escherParent.addChildRecord(escherTextbox);
+                    
+                    if ( shapeModel instanceof CommentShape ){
+                        CommentShape comment = (CommentShape)shapeModel;
+                        tailRec.add(comment.getNoteRecord());
+                    }
+
                 }
                 escherParent.addChildRecord( shapeModel.getSpContainer() );
             }
         }
+//        drawingManager.newCluster( (short)1 );
+//        drawingManager.newCluster( (short)2 );
+
     }
 
     private void convertGroup( HSSFShapeGroup shape, EscherContainerRecord escherParent, Map shapeToObj )
@@ -678,7 +850,7 @@ public class EscherAggregate extends AbstractEscherHolderRecord
         spContainer1.setRecordId( EscherContainerRecord.SP_CONTAINER );
         spContainer1.setOptions( (short) 0x000F );
         spgr.setRecordId( EscherSpgrRecord.RECORD_ID );
-        spgr.setOptions( (short) 0x0001 );    // don't know what the 1 is for.
+        spgr.setOptions( (short) 0x0001 );    // version
         spgr.setRectX1( patriarch.getX1() );
         spgr.setRectY1( patriarch.getY1() );
         spgr.setRectX2( patriarch.getX2() );
