@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -16,9 +17,9 @@
 
 package org.apache.poi.hssf.usermodel;
 
+import org.apache.poi.hssf.util.HSSFColor;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
-import org.apache.poi.hssf.util.HSSFColor;
 
 import java.awt.*;
 import java.awt.image.ImageObserver;
@@ -109,13 +110,13 @@ public class EscherGraphics
         this.verticalPixelsPerPoint = 1 / verticalPointsPerPixel;
     }
 
-    /**
-     * Constructs an escher graphics object.
-     *
-     * @param escherGroup           The escher group to write the graphics calls into.
-     * @param workbook              The workbook we are using.
-     * @param forecolor             The default foreground color.
-     */
+//    /**
+//     * Constructs an escher graphics object.
+//     *
+//     * @param escherGroup           The escher group to write the graphics calls into.
+//     * @param workbook              The workbook we are using.
+//     * @param forecolor             The default foreground color.
+//     */
 //    public EscherGraphics( HSSFShapeGroup escherGroup, HSSFWorkbook workbook, Color forecolor)
 //    {
 //        this(escherGroup, workbook, forecolor, 1.0f);
@@ -203,10 +204,15 @@ public class EscherGraphics
     }
 
     public void drawLine(int x1, int y1, int x2, int y2)
+    {
+        drawLine(x1,y1,x2,y2,0);
+    }
+
+    public void drawLine(int x1, int y1, int x2, int y2, int width)
     {
         HSSFSimpleShape shape = escherGroup.createShape(new HSSFChildAnchor(x1, y1, x2, y2) );
         shape.setShapeType(HSSFSimpleShape.OBJECT_TYPE_LINE);
-        shape.setLineWidth(0);
+        shape.setLineWidth(width);
         shape.setLineStyleColor(foreground.getRed(), foreground.getGreen(), foreground.getBlue());
     }
 
@@ -345,6 +351,25 @@ public class EscherGraphics
         shape.setLineStyleColor(foreground.getRed(), foreground.getGreen(), foreground.getBlue());
     }
 
+    /**
+     * Fills a (closed) polygon, as defined by a pair of arrays, which
+     *  hold the <i>x</i> and <i>y</i> coordinates.
+     * <p>
+     * This draws the polygon, with <code>nPoint</code> line segments.
+     * The first <code>nPoint&nbsp;-&nbsp;1</code> line segments are
+     *  drawn between sequential points 
+     *  (<code>xPoints[i],yPoints[i],xPoints[i+1],yPoints[i+1]</code>).
+     * The final line segment is a closing one, from the last point to 
+     *  the first (assuming they are different).
+     * <p>
+     * The area inside of the polygon is defined by using an
+     *  even-odd fill rule (also known as the alternating rule), and 
+     *  the area inside of it is filled.
+     * @param xPoints array of the <code>x</code> coordinates.
+     * @param yPoints array of the <code>y</code> coordinates.
+     * @param nPoints the total number of points in the polygon.
+     * @see   java.awt.Graphics#drawPolygon(int[], int[], int)
+     */
     public void fillPolygon(int xPoints[], int yPoints[],
 				     int nPoints)
     {
@@ -475,6 +500,9 @@ public class EscherGraphics
         this.background = background;
     }
 
-
+    HSSFShapeGroup getEscherGraphics()
+    {
+        return escherGroup;
+    }
 }
 
