@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -26,6 +27,7 @@ import java.awt.font.TextLayout;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Area;
 import java.awt.geom.GeneralPath;
+import java.awt.geom.Line2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.BufferedImageOp;
 import java.awt.image.ImageObserver;
@@ -141,8 +143,22 @@ public class EscherGraphics2d extends Graphics2D
 
     public void draw(Shape shape)
     {
-        if (logger.check( POILogger.WARN ))
-            logger.log(POILogger.WARN,"copyArea not supported");
+        if (shape instanceof Line2D)
+        {
+            Line2D shape2d = (Line2D) shape;
+
+            int width = 0;
+            if (stroke != null && stroke instanceof BasicStroke) {
+                width = (int) ((BasicStroke)stroke).getLineWidth() * 12700;
+            }
+
+            drawLine((int)shape2d.getX1(), (int)shape2d.getY1(), (int)shape2d.getX2(), (int)shape2d.getY2(), width);
+        }
+        else
+        {
+            if (logger.check(POILogger.WARN))
+                logger.log(POILogger.WARN, "draw not fully supported");
+        }
     }
 
     public void drawArc(int x, int y, int width, int height,
@@ -210,9 +226,18 @@ public class EscherGraphics2d extends Graphics2D
         drawImage(((Image) (img)), new AffineTransform(1.0F, 0.0F, 0.0F, 1.0F, x, y), null);
     }
 
+    public void drawLine(int x1, int y1, int x2, int y2, int width)
+    {
+        getEscherGraphics().drawLine(x1,y1,x2,y2, width);
+    }
+
     public void drawLine(int x1, int y1, int x2, int y2)
     {
-        getEscherGraphics().drawLine(x1,y1,x2,y2);
+        int width = 0;
+        if (stroke != null && stroke instanceof BasicStroke) {
+            width = (int) ((BasicStroke)stroke).getLineWidth() * 12700;
+        }
+        getEscherGraphics().drawLine(x1,y1,x2,y2, width);
 //        draw(new GeneralPath(new java.awt.geom.Line2D.Float(x1, y1, x2, y2)));
     }
 
@@ -304,24 +329,23 @@ public class EscherGraphics2d extends Graphics2D
     }
 
     /**
-     * Fills a closed polygon defined by
-     * arrays of <i>x</i> and <i>y</i> coordinates.
+     * Fills a (closed) polygon, as defined by a pair of arrays, which
+     *  hold the <i>x</i> and <i>y</i> coordinates.
      * <p>
-     * This method draws the polygon defined by <code>nPoint</code> line
-     * segments, where the first <code>nPoint&nbsp;-&nbsp;1</code>
-     * line segments are line segments from
-     * <code>(xPoints[i&nbsp;-&nbsp;1],&nbsp;yPoints[i&nbsp;-&nbsp;1])</code>
-     * to <code>(xPoints[i],&nbsp;yPoints[i])</code>, for
-     * 1&nbsp;&le;&nbsp;<i>i</i>&nbsp;&le;&nbsp;<code>nPoints</code>.
-     * The figure is automatically closed by drawing a line connecting
-     * the final point to the first point, if those points are different.
+     * This draws the polygon, with <code>nPoint</code> line segments.
+     * The first <code>nPoint&nbsp;-&nbsp;1</code> line segments are
+     *  drawn between sequential points 
+     *  (<code>xPoints[i],yPoints[i],xPoints[i+1],yPoints[i+1]</code>).
+     * The final line segment is a closing one, from the last point to 
+     *  the first (assuming they are different).
      * <p>
-     * The area inside the polygon is defined using an
-     * even-odd fill rule, also known as the alternating rule.
-     * @param        xPoints   a an array of <code>x</code> coordinates.
-     * @param        yPoints   a an array of <code>y</code> coordinates.
-     * @param        nPoints   a the total number of points.
-     * @see          java.awt.Graphics#drawPolygon(int[], int[], int)
+     * The area inside of the polygon is defined by using an
+     *  even-odd fill rule (also known as the alternating rule), and 
+     *  the area inside of it is filled.
+     * @param xPoints array of the <code>x</code> coordinates.
+     * @param yPoints array of the <code>y</code> coordinates.
+     * @param nPoints the total number of points in the polygon.
+     * @see   java.awt.Graphics#drawPolygon(int[], int[], int)
      */
     public void fillPolygon(int xPoints[], int yPoints[], int nPoints)
     {
