@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -23,42 +24,49 @@
  */
 package org.apache.poi.hssf.usermodel;
 
-import org.apache.poi.hssf.model.Workbook;
-import org.apache.poi.hssf.model.Sheet;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
+import java.util.Calendar;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.Iterator;
+
 import org.apache.poi.hssf.model.FormulaParser;
-import org.apache.poi.hssf.record.CellValueRecordInterface;
-import org.apache.poi.hssf.record.Record;
-import org.apache.poi.hssf.record.FormulaRecord;
-import org.apache.poi.hssf.record.LabelSSTRecord;
-import org.apache.poi.hssf.record.NumberRecord;
+import org.apache.poi.hssf.model.Sheet;
+import org.apache.poi.hssf.model.Workbook;
 import org.apache.poi.hssf.record.BlankRecord;
 import org.apache.poi.hssf.record.BoolErrRecord;
+import org.apache.poi.hssf.record.CellValueRecordInterface;
+import org.apache.poi.hssf.record.CommonObjectDataSubRecord;
 import org.apache.poi.hssf.record.ExtendedFormatRecord;
+import org.apache.poi.hssf.record.FormulaRecord;
+import org.apache.poi.hssf.record.LabelSSTRecord;
+import org.apache.poi.hssf.record.NoteRecord;
+import org.apache.poi.hssf.record.NumberRecord;
+import org.apache.poi.hssf.record.ObjRecord;
+import org.apache.poi.hssf.record.Record;
+import org.apache.poi.hssf.record.SubRecord;
+import org.apache.poi.hssf.record.TextObjectRecord;
+import org.apache.poi.hssf.record.UnicodeString;
 import org.apache.poi.hssf.record.aggregates.FormulaRecordAggregate;
 import org.apache.poi.hssf.record.formula.Ptg;
-import org.apache.poi.hssf.util.SheetReferences;
-
-//import org.apache.poi.hssf.record.formula.FormulaParser;
-
-import java.util.Date;
-import java.util.Calendar;
 
 /**
  * High level representation of a cell in a row of a spreadsheet.
  * Cells can be numeric, formula-based or string-based (text).  The cell type
  * specifies this.  String cells cannot conatin numbers and numeric cells cannot
  * contain strings (at least according to our model).  Client apps should do the
- * conversions themselves.  Formula cells are treated like string cells, simply
- * containing a formula string.  They'll be rendered differently.
+ * conversions themselves.  Formula cells have the formula string, as well as 
+ * the formula result, which can be numeric or string. 
  * <p>
  * Cells should have their number (0 based) before being added to a row.  Only
  * cells that have values should be added.
  * <p>
- * NOTE: the alpha won't be implementing formulas
  *
  * @author  Andrew C. Oliver (acoliver at apache dot org)
  * @author  Dan Sherman (dsherman at isisph.com)
  * @author  Brian Sanders (kestrel at burdell dot org) Active Cell support
+ * @author  Yegor Kozlov cell comments support
  * @version 1.0-pre
  */
 
@@ -112,21 +120,16 @@ public class HSSFCell
      */
 
     public final static int          CELL_TYPE_ERROR             = 5;
+    public final static short        ENCODING_UNCHANGED          = -1;
     public final static short        ENCODING_COMPRESSED_UNICODE = 0;
     public final static short        ENCODING_UTF_16             = 1;
-    private short                    cellNum;
     private int                      cellType;
-    private HSSFCellStyle            cellStyle;
-    private double                   cellValue;
-    private String                   stringValue;
-    private boolean                  booleanValue;
-    private byte                     errorValue;
-    private short                    encoding = ENCODING_COMPRESSED_UNICODE;
+    private HSSFRichTextString       stringValue;
+    private short                    encoding = ENCODING_UNCHANGED;
     private Workbook                 book;
     private Sheet                    sheet;
-    //private short                    row;
-    private int                    row;
     private CellValueRecordInterface record;
+    private HSSFComment              comment;
 
     /**
      * Creates new Cell - Should only be called by HSSFRow.  This creates a cell
@@ -149,23 +152,15 @@ public class HSSFCell
     protected HSSFCell(Workbook book, Sheet sheet, int row, short col)
     {
         checkBounds(col);
-        cellNum      = col;
-        this.row     = row;
-        cellStyle    = null;
-        cellValue    = 0;
         stringValue  = null;
-        booleanValue = false;
-        errorValue   = ( byte ) 0;
         this.book    = book;
         this.sheet   = sheet;
 
         // Relying on the fact that by default the cellType is set to 0 which
         // is different to CELL_TYPE_BLANK hence the following method call correctly
         // creates a new blank cell.
-        setCellType(CELL_TYPE_BLANK, false);
-        ExtendedFormatRecord xf = book.getExFormatAt(0xf);
-
-        setCellStyle(new HSSFCellStyle(( short ) 0xf, xf));
+        short xfindex = sheet.getXFIndexForColAt(col);
+        setCellType(CELL_TYPE_BLANK, false, row, col,xfindex);
     }
 
     /**
@@ -180,8 +175,6 @@ public class HSSFCell
      *                CELL_TYPE_BOOLEAN, CELL_TYPE_ERROR
      *                Type of cell
      * @see org.apache.poi.hssf.usermodel.HSSFRow#createCell(short,int)
-     * @deprecated As of 22-Jan-2002 use @see org.apache.poi.hssf.usermodel.HSSFRow#createCell(short)
-     * and use setCellValue to specify the type lazily.
      */
 
     //protected HSSFCell(Workbook book, Sheet sheet, short row, short col,
@@ -189,66 +182,13 @@ public class HSSFCell
                        int type)
     {
         checkBounds(col);
-        cellNum      = col;
-        this.row     = row;
-        cellType     = type;
-        cellStyle    = null;
-        cellValue    = 0;
+        cellType     = -1; // Force 'setCellType' to create a first Record
         stringValue  = null;
-        booleanValue = false;
-        errorValue   = ( byte ) 0;
         this.book    = book;
         this.sheet   = sheet;
-        switch (type)
-        {
-
-            case CELL_TYPE_NUMERIC :
-                record = new NumberRecord();
-                (( NumberRecord ) record).setColumn(col);
-                (( NumberRecord ) record).setRow(row);
-                (( NumberRecord ) record).setValue(( short ) 0);
-                (( NumberRecord ) record).setXFIndex(( short ) 0);
-                break;
-
-            case CELL_TYPE_STRING :
-                record = new LabelSSTRecord();
-                (( LabelSSTRecord ) record).setColumn(col);
-                (( LabelSSTRecord ) record).setRow(row);
-                (( LabelSSTRecord ) record).setXFIndex(( short ) 0);
-                break;
-
-            case CELL_TYPE_BLANK :
-                record = new BlankRecord();
-                (( BlankRecord ) record).setColumn(col);
-                (( BlankRecord ) record).setRow(row);
-                (( BlankRecord ) record).setXFIndex(( short ) 0);
-                break;
-
-            case CELL_TYPE_FORMULA :
-                FormulaRecord formulaRecord = new FormulaRecord();
-                record = new FormulaRecordAggregate(formulaRecord,null);
-                formulaRecord.setColumn(col);
-                formulaRecord.setRow(row);
-                formulaRecord.setXFIndex(( short ) 0);
-            case CELL_TYPE_BOOLEAN :
-                record = new BoolErrRecord();
-                (( BoolErrRecord ) record).setColumn(col);
-                (( BoolErrRecord ) record).setRow(row);
-                (( BoolErrRecord ) record).setXFIndex(( short ) 0);
-                (( BoolErrRecord ) record).setValue(false);
-                break;
-
-            case CELL_TYPE_ERROR :
-                record = new BoolErrRecord();
-                (( BoolErrRecord ) record).setColumn(col);
-                (( BoolErrRecord ) record).setRow(row);
-                (( BoolErrRecord ) record).setXFIndex(( short ) 0);
-                (( BoolErrRecord ) record).setValue(( byte ) 0);
-                break;
-        }
-        ExtendedFormatRecord xf = book.getExFormatAt(0xf);
-
-        setCellStyle(new HSSFCellStyle(( short ) 0xf, xf));
+        
+        short xfindex = sheet.getXFIndexForColAt(col);
+        setCellType(type,false,row,col,xfindex);
     }
 
     /**
@@ -264,40 +204,22 @@ public class HSSFCell
     protected HSSFCell(Workbook book, Sheet sheet, int row,
                        CellValueRecordInterface cval)
     {
-        cellNum     = cval.getColumn();
         record      = cval;
-        this.row    = row;
         cellType    = determineType(cval);
-        cellStyle   = null;
         stringValue = null;
         this.book   = book;
         this.sheet  = sheet;
         switch (cellType)
         {
-
-            case CELL_TYPE_NUMERIC :
-                cellValue = (( NumberRecord ) cval).getValue();
-                break;
-
             case CELL_TYPE_STRING :
-                stringValue =
-                    book.getSSTString( ( (LabelSSTRecord ) cval).getSSTIndex());
+                stringValue = new HSSFRichTextString(book, (LabelSSTRecord ) cval);
                 break;
 
             case CELL_TYPE_BLANK :
                 break;
 
             case CELL_TYPE_FORMULA :
-                cellValue = (( FormulaRecordAggregate ) cval).getFormulaRecord().getValue();
-                stringValue=((FormulaRecordAggregate) cval).getStringValue();
-                break;
-
-            case CELL_TYPE_BOOLEAN :
-                booleanValue = (( BoolErrRecord ) cval).getBooleanValue();
-                break;
-
-            case CELL_TYPE_ERROR :
-                errorValue = (( BoolErrRecord ) cval).getErrorValue();
+                stringValue=new HSSFRichTextString(((FormulaRecordAggregate) cval).getStringValue());
                 break;
         }
         ExtendedFormatRecord xf = book.getExFormatAt(cval.getXFIndex());
@@ -350,17 +272,34 @@ public class HSSFCell
         }
         return retval;
     }
+    
+    /**
+     * Returns the Workbook that this Cell is bound to
+     * @return
+     */
+    protected Workbook getBoundWorkbook() {
+    	return book;
+    }
 
     /**
-     * set the cell's number within the row (0 based)
+     * Set the cell's number within the row (0 based).
      * @param num  short the cell number
+     * @deprecated Doesn't update the row's idea of what cell this is, use {@link HSSFRow#moveCell(HSSFCell, short)} instead
      */
-
     public void setCellNum(short num)
     {
-        cellNum = num;
         record.setColumn(num);
     }
+    
+    /**
+     * Updates the cell record's idea of what
+     *  column it belongs in (0 based)
+     * @param num the new cell number
+     */
+    protected void updateCellNum(short num)
+    {
+    	record.setColumn(num);
+    }
 
     /**
      *  get the cell's number within the row
@@ -369,12 +308,11 @@ public class HSSFCell
 
     public short getCellNum()
     {
-        return cellNum;
+        return record.getColumn();
     }
 
     /**
-     * set the cells type (numeric, formula or string) -- DONT USE FORMULAS IN THIS RELEASE
-     * WE'LL THROW YOU A RUNTIME EXCEPTION IF YOU DO
+     * set the cells type (numeric, formula or string)
      * @see #CELL_TYPE_NUMERIC
      * @see #CELL_TYPE_STRING
      * @see #CELL_TYPE_FORMULA
@@ -385,7 +323,10 @@ public class HSSFCell
 
     public void setCellType(int cellType)
     {
-        setCellType(cellType, true);
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
+        setCellType(cellType, true, row, col, styleIndex);
     }
 
     /**
@@ -397,7 +338,7 @@ public class HSSFCell
      *
      */
 
-    private void setCellType(int cellType, boolean setValue)
+    private void setCellType(int cellType, boolean setValue, int row,short col, short styleIndex)
     {
 
         // if (cellType == CELL_TYPE_FORMULA)
@@ -423,12 +364,12 @@ public class HSSFCell
                 {
                     frec = ( FormulaRecordAggregate ) record;
                 }
-                frec.setColumn(getCellNum());
+                frec.setColumn(col);
                 if (setValue)
                 {
                     frec.getFormulaRecord().setValue(getNumericCellValue());
                 }
-                frec.setXFIndex(( short ) cellStyle.getIndex());
+                frec.setXFIndex(styleIndex);
                 frec.setRow(row);
                 record = frec;
                 break;
@@ -444,12 +385,12 @@ public class HSSFCell
                 {
                     nrec = ( NumberRecord ) record;
                 }
-                nrec.setColumn(getCellNum());
+                nrec.setColumn(col);
                 if (setValue)
                 {
                     nrec.setValue(getNumericCellValue());
                 }
-                nrec.setXFIndex(( short ) cellStyle.getIndex());
+                nrec.setXFIndex(styleIndex);
                 nrec.setRow(row);
                 record = nrec;
                 break;
@@ -465,9 +406,9 @@ public class HSSFCell
                 {
                     lrec = ( LabelSSTRecord ) record;
                 }
-                lrec.setColumn(getCellNum());
+                lrec.setColumn(col);
                 lrec.setRow(row);
-                lrec.setXFIndex(( short ) cellStyle.getIndex());
+                lrec.setXFIndex(styleIndex);
                 if (setValue)
                 {
                     if ((getStringCellValue() != null)
@@ -475,16 +416,17 @@ public class HSSFCell
                     {
                         int sst = 0;
 
-                        if (encoding == ENCODING_COMPRESSED_UNICODE)
-                        {
-                            sst = book.addSSTString(getStringCellValue());
-                        }
-                        if (encoding == ENCODING_UTF_16)
-                        {
-                            sst = book.addSSTString(getStringCellValue(),
-                                                    true);
-                        }
+                        UnicodeString str = getRichStringCellValue().getUnicodeString();
+//jmh                        if (encoding == ENCODING_COMPRESSED_UNICODE)
+//jmh                        {
+//                      jmh                            str.setCompressedUnicode();
+//                      jmh                        } else if (encoding == ENCODING_UTF_16)
+//                      jmh                        {
+//                      jmh                            str.setUncompressedUnicode();
+//                      jmh                        }
+                        sst = book.addSSTString(str);
                         lrec.setSSTIndex(sst);
+                        getRichStringCellValue().setUnicodeString(book.getSSTString(sst));
                     }
                 }
                 record = lrec;
@@ -501,17 +443,10 @@ public class HSSFCell
                 {
                     brec = ( BlankRecord ) record;
                 }
-                brec.setColumn(getCellNum());
+                brec.setColumn(col);
 
                 // During construction the cellStyle may be null for a Blank cell.
-                if (cellStyle != null)
-                {
-                    brec.setXFIndex(( short ) cellStyle.getIndex());
-                }
-                else
-                {
-                    brec.setXFIndex(( short ) 0);
-                }
+                brec.setXFIndex(styleIndex);
                 brec.setRow(row);
                 record = brec;
                 break;
@@ -527,12 +462,12 @@ public class HSSFCell
                 {
                     boolRec = ( BoolErrRecord ) record;
                 }
-                boolRec.setColumn(getCellNum());
+                boolRec.setColumn(col);
                 if (setValue)
                 {
                     boolRec.setValue(getBooleanCellValue());
                 }
-                boolRec.setXFIndex(( short ) cellStyle.getIndex());
+                boolRec.setXFIndex(styleIndex);
                 boolRec.setRow(row);
                 record = boolRec;
                 break;
@@ -548,17 +483,18 @@ public class HSSFCell
                 {
                     errRec = ( BoolErrRecord ) record;
                 }
-                errRec.setColumn(getCellNum());
+                errRec.setColumn(col);
                 if (setValue)
                 {
                     errRec.setValue(getErrorCellValue());
                 }
-                errRec.setXFIndex(( short ) cellStyle.getIndex());
+                errRec.setXFIndex(styleIndex);
                 errRec.setRow(row);
                 record = errRec;
                 break;
         }
-        if (cellType != this.cellType)
+        if (cellType != this.cellType && 
+            this.cellType!=-1 )  // Special Value to indicate an uninitialized Cell
         {
             int loc = sheet.getLoc();
 
@@ -591,12 +527,20 @@ public class HSSFCell
      */
     public void setCellValue(double value)
     {
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
         if ((cellType != CELL_TYPE_NUMERIC) && (cellType != CELL_TYPE_FORMULA))
         {
-            setCellType(CELL_TYPE_NUMERIC, false);
+            setCellType(CELL_TYPE_NUMERIC, false, row, col, styleIndex);
+        }
+        
+        // Save into the apropriate record
+        if(record instanceof FormulaRecordAggregate) {
+        	(( FormulaRecordAggregate ) record).getFormulaRecord().setValue(value);
+        } else {
+        	(( NumberRecord ) record).setValue(value);
         }
-        (( NumberRecord ) record).setValue(value);
-        cellValue = value;
     }
 
     /**
@@ -609,7 +553,7 @@ public class HSSFCell
      */
     public void setCellValue(Date value)
     {
-        setCellValue(HSSFDateUtil.getExcelDate(value));
+        setCellValue(HSSFDateUtil.getExcelDate(value, this.book.isUsing1904DateWindowing()));
     }
 
     /**
@@ -633,44 +577,71 @@ public class HSSFCell
      * string, for String cells we'll set its value.  For other types we will
      * change the cell to a string cell and set its value.
      * If value is null then we will change the cell to a Blank cell.
+     * @deprecated Use setCellValue(HSSFRichTextString) instead.
      */
 
     public void setCellValue(String value)
     {
+      HSSFRichTextString str = new HSSFRichTextString(value);
+      setCellValue(str);
+    }
+
+    /**
+     * set a string value for the cell. Please note that if you are using
+     * full 16 bit unicode you should call <code>setEncoding()</code> first.
+     *
+     * @param value  value to set the cell to.  For formulas we'll set the formula
+     * string, for String cells we'll set its value.  For other types we will
+     * change the cell to a string cell and set its value.
+     * If value is null then we will change the cell to a Blank cell.
+     */
+
+    public void setCellValue(HSSFRichTextString value)
+    {
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
         if (value == null)
         {
-            setCellType(CELL_TYPE_BLANK, false);
+            setCellType(CELL_TYPE_BLANK, false, row, col, styleIndex);
         }
         else
         {
             if ((cellType != CELL_TYPE_STRING ) && ( cellType != CELL_TYPE_FORMULA))
             {
-                setCellType(CELL_TYPE_STRING, false);
+                setCellType(CELL_TYPE_STRING, false, row, col, styleIndex);
             }
             int index = 0;
 
-            if (encoding == ENCODING_COMPRESSED_UNICODE)
-            {
-                index = book.addSSTString(value);
-            }
-            if (encoding == ENCODING_UTF_16)
-            {
-                index = book.addSSTString(value, true);
-            }
+            UnicodeString str = value.getUnicodeString();            
+//          jmh            if (encoding == ENCODING_COMPRESSED_UNICODE)
+//          jmh            {
+//          jmh                str.setCompressedUnicode();
+//          jmh            } else if (encoding == ENCODING_UTF_16)
+//          jmh            {
+//          jmh                str.setUncompressedUnicode();
+//          jmh            }
+            index = book.addSSTString(str);            
             (( LabelSSTRecord ) record).setSSTIndex(index);
             stringValue = value;
+            stringValue.setWorkbookReferences(book, (( LabelSSTRecord ) record));
+            stringValue.setUnicodeString(book.getSSTString(index));            
         }
     }
 
     public void setCellFormula(String formula) {
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
         //Workbook.currentBook=book;
         if (formula==null) {
-            setCellType(CELL_TYPE_BLANK,false);
+            setCellType(CELL_TYPE_BLANK,false,row,col,styleIndex);
         } else {
-            setCellType(CELL_TYPE_FORMULA,false);
+            setCellType(CELL_TYPE_FORMULA,false,row,col,styleIndex);
             FormulaRecordAggregate rec = (FormulaRecordAggregate) record;
-            rec.getFormulaRecord().setOptions(( short ) 2);
-            rec.getFormulaRecord().setValue(0);
+            FormulaRecord frec = rec.getFormulaRecord();
+            frec.setOptions(( short ) 2);
+            frec.setValue(0);
             
             //only set to default if there is no extended format index already set
             if (rec.getXFIndex() == (short)0) rec.setXFIndex(( short ) 0x0f);
@@ -678,10 +649,16 @@ public class HSSFCell
             fp.parse();
             Ptg[] ptg  = fp.getRPNPtg();
             int   size = 0;
-            //System.out.println("got Ptgs " + ptg.length);
+
+            // clear the Ptg Stack
+            for (int i=0, iSize=frec.getNumberOfExpressionTokens(); i<iSize; i++) {
+                frec.popExpressionToken();
+            }
+
+            // fill the Ptg Stack with Ptgs of new formula
             for (int k = 0; k < ptg.length; k++) {
                 size += ptg[ k ].getSize();
-                rec.getFormulaRecord().pushExpressionToken(ptg[ k ]);
+                frec.pushExpressionToken(ptg[ k ]);
             }
             rec.getFormulaRecord().setExpressionLength(( short ) size);
             //Workbook.currentBook = null;
@@ -722,7 +699,15 @@ public class HSSFCell
             throw new NumberFormatException(
                 "You cannot get a numeric value from an error cell");
         }
-        return cellValue;
+        if(cellType == CELL_TYPE_NUMERIC)
+        {
+          return ((NumberRecord)record).getValue();
+        }
+        if(cellType == CELL_TYPE_FORMULA)
+        {
+          return ((FormulaRecordAggregate)record).getFormulaRecord().getValue();
+        }
+        throw new NumberFormatException("Unknown Record Type in Cell:"+cellType);
     }
 
     /**
@@ -750,25 +735,39 @@ public class HSSFCell
             throw new NumberFormatException(
                 "You cannot get a date value from an error cell");
         }
+        double value=this.getNumericCellValue();
         if (book.isUsing1904DateWindowing()) {
-            return HSSFDateUtil.getJavaDate(cellValue,true);
+            return HSSFDateUtil.getJavaDate(value,true);
         }
         else {
-            return HSSFDateUtil.getJavaDate(cellValue,false);
+            return HSSFDateUtil.getJavaDate(value,false);
         }
     }
 
+    /**
+     * get the value of the cell as a string - for numeric cells we throw an exception.
+     * For blank cells we return an empty string.
+     * For formulaCells that are not string Formulas, we return empty String
+     * @deprecated Use the HSSFRichTextString return
+     */
+
+    public String getStringCellValue()
+    {
+      HSSFRichTextString str = getRichStringCellValue();
+      return str.getString();
+    }
+
     /**
      * get the value of the cell as a string - for numeric cells we throw an exception.
      * For blank cells we return an empty string.
      * For formulaCells that are not string Formulas, we return empty String
      */
 
-    public String getStringCellValue()
+    public HSSFRichTextString getRichStringCellValue()
     {
         if (cellType == CELL_TYPE_BLANK)
         {
-            return "";
+            return new HSSFRichTextString("");
         }
         if (cellType == CELL_TYPE_NUMERIC)
         {
@@ -787,7 +786,7 @@ public class HSSFCell
         }
         if (cellType == CELL_TYPE_FORMULA) 
         {
-            if (stringValue==null) return "";
+            if (stringValue==null) return new HSSFRichTextString("");
         }
         return stringValue;
     }
@@ -802,12 +801,14 @@ public class HSSFCell
 
     public void setCellValue(boolean value)
     {
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
         if ((cellType != CELL_TYPE_BOOLEAN ) && ( cellType != CELL_TYPE_FORMULA))
         {
-            setCellType(CELL_TYPE_BOOLEAN, false);
+            setCellType(CELL_TYPE_BOOLEAN, false, row, col, styleIndex);
         }
         (( BoolErrRecord ) record).setValue(value);
-        booleanValue = value;
     }
 
     /**
@@ -821,12 +822,14 @@ public class HSSFCell
 
     public void setCellErrorValue(byte value)
     {
+        int row=record.getRow();
+        short col=record.getColumn();
+        short styleIndex=record.getXFIndex();
         if ((cellType != CELL_TYPE_ERROR) && (cellType != CELL_TYPE_FORMULA))
         {
-            setCellType(CELL_TYPE_ERROR, false);
+            setCellType(CELL_TYPE_ERROR, false, row, col, styleIndex);
         }
         (( BoolErrRecord ) record).setValue(value);
-        errorValue = value;
     }
 
     /**
@@ -838,7 +841,7 @@ public class HSSFCell
     {
         if (cellType == CELL_TYPE_BOOLEAN)
         {
-            return booleanValue;
+            return (( BoolErrRecord ) record).getBooleanValue();
         }
         if (cellType == CELL_TYPE_BLANK)
         {
@@ -857,7 +860,7 @@ public class HSSFCell
     {
         if (cellType == CELL_TYPE_ERROR)
         {
-            return errorValue;
+            return (( BoolErrRecord ) record).getErrorValue();
         }
         if (cellType == CELL_TYPE_BLANK)
         {
@@ -878,7 +881,6 @@ public class HSSFCell
 
     public void setCellStyle(HSSFCellStyle style)
     {
-        cellStyle = style;
         record.setXFIndex(style.getIndex());
     }
 
@@ -890,18 +892,22 @@ public class HSSFCell
 
     public HSSFCellStyle getCellStyle()
     {
-        return cellStyle;
+      short styleIndex=record.getXFIndex();
+      ExtendedFormatRecord xf = book.getExFormatAt(styleIndex);
+      return new HSSFCellStyle(styleIndex, xf);
     }
 
     /**
-     * used for internationalization, currently 0 for compressed unicode or 1 for 16-bit
+     * used for internationalization, currently -1 for unchanged, 0 for compressed unicode or 1 for 16-bit
      *
+     * @see #ENCODING_UNCHANGED
      * @see #ENCODING_COMPRESSED_UNICODE
      * @see #ENCODING_UTF_16
      *
-     * @return 1 or 0 for compressed or uncompressed (used only with String type)
+     * @return -1, 1 or 0 for unchanged, compressed or uncompressed (used only with String type)
+     * 
+     * @deprecated As of 3-Jan-06 POI now automatically handles Unicode without forcing the encoding.
      */
-
     public short getEncoding()
     {
         return encoding;
@@ -910,10 +916,12 @@ public class HSSFCell
     /**
      * set the encoding to either 8 or 16 bit. (US/UK use 8-bit, rest of the western world use 16bit)
      *
+     * @see #ENCODING_UNCHANGED
      * @see #ENCODING_COMPRESSED_UNICODE
      * @see #ENCODING_UTF_16
      *
      * @param encoding either ENCODING_COMPRESSED_UNICODE (0) or ENCODING_UTF_16 (1)
+     * @deprecated As of 3-Jan-06 POI now automatically handles Unicode without forcing the encoding.
      */
 
     public void setEncoding(short encoding)
@@ -950,7 +958,114 @@ public class HSSFCell
      */
     public void setAsActiveCell()
     {
-        this.sheet.setActiveCellRow(this.row);
-        this.sheet.setActiveCellCol(this.cellNum);
+        int row=record.getRow();
+        short col=record.getColumn();
+        this.sheet.setActiveCellRow(row);
+        this.sheet.setActiveCellCol(col);
     }
+    
+    /**
+     * Returns a string representation of the cell
+     * 
+     * This method returns a simple representation, 
+     * anthing more complex should be in user code, with
+     * knowledge of the semantics of the sheet being processed. 
+     * 
+     * Formula cells return the formula string, 
+     * rather than the formula result. 
+     * Dates are displayed in dd-MMM-yyyy format
+     * Errors are displayed as #ERR&lt;errIdx&gt;
+     */
+    public String toString() {
+    	switch     (getCellType()) {
+    		case CELL_TYPE_BLANK:
+    			return "";
+    		case CELL_TYPE_BOOLEAN:
+    			return getBooleanCellValue()?"TRUE":"FALSE";
+    		case CELL_TYPE_ERROR:
+    			return "#ERR"+getErrorCellValue();
+    		case CELL_TYPE_FORMULA:
+    			return getCellFormula();
+    		case CELL_TYPE_NUMERIC:
+    			//TODO apply the dataformat for this cell
+    			if (HSSFDateUtil.isCellDateFormatted(this)) {
+    				DateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
+    				return sdf.format(getDateCellValue());
+    			}else {
+    				return  getNumericCellValue() + "";
+    			}
+    		case CELL_TYPE_STRING:
+    			return getStringCellValue();
+    		default:
+    			return "Unknown Cell Type: " + getCellType();
+    	}
+    }
+
+    /**
+     * Assign a comment to this cell
+     *
+     * @param comment comment associated with this cell
+     */
+    public void setCellComment(HSSFComment comment){
+        comment.setRow((short)record.getRow());
+        comment.setColumn(record.getColumn());
+        this.comment = comment;
+    }
+
+    /**
+     * Returns comment associated with this cell
+     *
+     * @return comment associated with this cell
+     */
+     public HSSFComment getCellComment(){
+        if (comment == null) {
+            comment = findCellComment(sheet, record.getRow(), record.getColumn());
+        }
+        return comment;
+    }
+
+    /**
+     * Cell comment finder.
+     * Returns cell comment for the specified sheet, row and column.
+     *
+     * @return cell comment or <code>null</code> if not found
+     */
+    protected static HSSFComment findCellComment(Sheet sheet, int row, int column){
+        HSSFComment comment = null;
+        HashMap txshapes = new HashMap(); //map shapeId and TextObjectRecord
+        for (Iterator it = sheet.getRecords().iterator(); it.hasNext(); ) {
+           Record rec = ( Record ) it.next();
+           if (rec instanceof NoteRecord){
+               NoteRecord note = (NoteRecord)rec;
+               if (note.getRow() == row && note.getColumn() == column){
+                   TextObjectRecord txo = (TextObjectRecord)txshapes.get(new Integer(note.getShapeId()));
+                   comment = new HSSFComment(note, txo);
+                   comment.setRow(note.getRow());
+                   comment.setColumn(note.getColumn());
+                   comment.setAuthor(note.getAuthor());
+                   comment.setVisible(note.getFlags() == NoteRecord.NOTE_VISIBLE);
+                   comment.setString(txo.getStr());
+                   break;
+               }
+           } else if (rec instanceof ObjRecord){
+               ObjRecord obj = (ObjRecord)rec;
+               SubRecord sub = (SubRecord)obj.getSubRecords().get(0);
+               if (sub instanceof CommonObjectDataSubRecord){
+                   CommonObjectDataSubRecord cmo = (CommonObjectDataSubRecord)sub;
+                   if (cmo.getObjectType() == CommonObjectDataSubRecord.OBJECT_TYPE_COMMENT){
+                       //find the nearest TextObjectRecord which holds comment's text and map it to its shapeId
+                       while(it.hasNext()) {
+                           rec = ( Record ) it.next();
+                           if (rec instanceof TextObjectRecord) {
+                               txshapes.put(new Integer(cmo.getObjectId()), rec);
+                               break;
+                           }
+                       }
+
+                   }
+               }
+           }
+        }
+        return comment;
+   }
 }
