@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -17,7 +18,10 @@
 
 package org.apache.poi.hssf.usermodel;
 
+import org.apache.poi.hssf.model.Workbook;
 import org.apache.poi.hssf.record.ExtendedFormatRecord;
+import org.apache.poi.hssf.record.FormatRecord;
+import org.apache.poi.hssf.util.*;
 
 /**
  * High level representation of the style of a cell in a sheet of a workbook.
@@ -35,7 +39,6 @@ public class HSSFCellStyle
 {
     private ExtendedFormatRecord format                     = null;
     private short                index                      = 0;
-    private short                fontindex                  = 0;
 
     /**
      * general (normal) horizontal alignment
@@ -221,6 +224,10 @@ public class HSSFCellStyle
     public final static short     SQUARES             = 15 ;
     /**  Diamonds */
     public final static short     DIAMONDS            = 16 ;
+    /**  Less Dots */
+    public final static short     LESS_DOTS           = 17 ;
+    /**  Least Dots */
+    public final static short     LEAST_DOTS          = 18 ;
 
 
     /** Creates new HSSFCellStyle why would you want to do this?? */
@@ -261,6 +268,17 @@ public class HSSFCellStyle
     {
         return format.getFormatIndex();
     }
+    
+    /**
+     * Get the contents of the format string, by looking up
+     *  the DataFormat against the supplied workbook
+     * @see org.apache.poi.hssf.usermodel.HSSFDataFormat
+     */
+    public String getDataFormatString(Workbook workbook) {
+    	HSSFDataFormat format = new HSSFDataFormat(workbook);
+    	
+        return format.getFormat(getDataFormat());
+    }
 
     /**
      * set the font for this style
@@ -272,14 +290,28 @@ public class HSSFCellStyle
     public void setFont(HSSFFont font)
     {
         format.setIndentNotParentFont(true);
-        fontindex = font.getIndex();
+        short fontindex = font.getIndex();
         format.setFontIndex(fontindex);
     }
 
+    /**
+     * gets the index of the font for this style
+     * @see org.apache.poi.hssf.usermodel.HSSFWorkbook#getFontAt(short)
+     */
     public short getFontIndex()
     {
         return format.getFontIndex();
     }
+    
+    /**
+     * gets the font for this style
+     * @param parentWorkbook The HSSFWorkbook that this style belongs to
+     * @see org.apache.poi.hssf.usermodel.HSSFCellStyle#getFontIndex()
+     * @see org.apache.poi.hssf.usermodel.HSSFWorkbook#getFontAt(short)
+     */
+    public HSSFFont getFont(HSSFWorkbook parentWorkbook) {
+    	return parentWorkbook.getFontAt(getFontIndex());
+    }
 
     /**
      * set the cell's using this style to be hidden
@@ -671,7 +703,6 @@ public class HSSFCellStyle
      * @see #BORDER_MEDIUM_DASH_DOT_DOT
      * @see #BORDER_SLANTED_DASH_DOT
      */
-
     public short getBorderBottom()
     {
         return format.getBorderBottom();
@@ -679,9 +710,8 @@ public class HSSFCellStyle
 
     /**
      * set the color to use for the left border
-     * @param color
+     * @param color The index of the color definition
      */
-
     public void setLeftBorderColor(short color)
     {
         format.setLeftBorderPaletteIdx(color);
@@ -689,9 +719,9 @@ public class HSSFCellStyle
 
     /**
      * get the color to use for the left border
-     * @return color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
+     * @param color The index of the color definition
      */
-
     public short getLeftBorderColor()
     {
         return format.getLeftBorderPaletteIdx();
@@ -699,9 +729,8 @@ public class HSSFCellStyle
 
     /**
      * set the color to use for the right border
-     * @param color
+     * @param color The index of the color definition
      */
-
     public void setRightBorderColor(short color)
     {
         format.setRightBorderPaletteIdx(color);
@@ -709,9 +738,9 @@ public class HSSFCellStyle
 
     /**
      * get the color to use for the left border
-     * @return color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
+     * @param color The index of the color definition
      */
-
     public short getRightBorderColor()
     {
         return format.getRightBorderPaletteIdx();
@@ -719,9 +748,8 @@ public class HSSFCellStyle
 
     /**
      * set the color to use for the top border
-     * @param color
+     * @param color The index of the color definition
      */
-
     public void setTopBorderColor(short color)
     {
         format.setTopBorderPaletteIdx(color);
@@ -729,9 +757,9 @@ public class HSSFCellStyle
 
     /**
      * get the color to use for the top border
-     * @return color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
+     * @param color The index of the color definition
      */
-
     public short getTopBorderColor()
     {
         return format.getTopBorderPaletteIdx();
@@ -739,9 +767,8 @@ public class HSSFCellStyle
 
     /**
      * set the color to use for the bottom border
-     * @param color
+     * @param color The index of the color definition
      */
-
     public void setBottomBorderColor(short color)
     {
         format.setBottomBorderPaletteIdx(color);
@@ -749,9 +776,9 @@ public class HSSFCellStyle
 
     /**
      * get the color to use for the left border
-     * @return color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
+     * @param color The index of the color definition
      */
-
     public short getBottomBorderColor()
     {
         return format.getBottomBorderPaletteIdx();
@@ -795,6 +822,29 @@ public class HSSFCellStyle
     {
         return format.getAdtlFillPattern();
     }
+    
+    /**
+     * Checks if the background and foreground fills are set correctly when one
+     * or the other is set to the default color.
+     * <p>Works like the logic table below:</p>
+     * <p>BACKGROUND   FOREGROUND</p>
+     * <p>NONE         AUTOMATIC</p>
+     * <p>0x41         0x40</p>
+     * <p>NONE         RED/ANYTHING</p>
+     * <p>0x40         0xSOMETHING</p> 
+     */
+    private void checkDefaultBackgroundFills() {
+      if (format.getFillForeground() == org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index) {
+    	  //JMH: Why +1, hell why not. I guess it made some sense to someone at the time. Doesnt
+    	  //to me now.... But experience has shown that when the fore is set to AUTOMATIC then the
+    	  //background needs to be incremented......
+    	  if (format.getFillBackground() != (org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index+1))
+    		  setFillBackgroundColor((short)(org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index+1));
+      } else if (format.getFillBackground() == org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index+1)
+    	  //Now if the forground changes to a non-AUTOMATIC color the background resets itself!!!
+    	  if (format.getFillForeground() != org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index)
+    		  setFillBackgroundColor(org.apache.poi.hssf.util.HSSFColor.AUTOMATIC.index);
+    }
 
     /**
      * set the background fill color.
@@ -802,12 +852,19 @@ public class HSSFCellStyle
      * For example:
      * <pre>
      * cs.setFillPattern(HSSFCellStyle.FINE_DOTS );
-     * cs.setFillBackgroundColor(HSSFCellStyle.RED);
+     * cs.setFillBackgroundColor(new HSSFColor.RED().getIndex()); 
+     * </pre>
+     * optionally a Foreground and background fill can be applied:
+     * <i>Note: Ensure Foreground color is set prior to background</i>
+     * <pre>
+     * cs.setFillPattern(HSSFCellStyle.FINE_DOTS );
+     * cs.setFillForegroundColor(new HSSFColor.BLUE().getIndex());
+     * cs.setFillBackgroundColor(new HSSFColor.RED().getIndex()); 
      * </pre>
      * or, for the special case of SOLID_FILL:
      * <pre>
-     * cs.setFillPattern(HSSFCellStyle.SOLID_FILL );
-     * cs.setFillForgroundColor(HSSFSeCellStyle.RED);
+     * cs.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND );
+     * cs.setFillForegroundColor(new HSSFColor.RED().getIndex());
      * </pre>
      * It is necessary to set the fill style in order
      * for the color to be shown in the cell.
@@ -816,38 +873,44 @@ public class HSSFCellStyle
      */
 
     public void setFillBackgroundColor(short bg)
-    {
+    {    	
         format.setFillBackground(bg);
+        checkDefaultBackgroundFills();
     }
 
     /**
      * get the background fill color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
      * @return fill color
      */
-
     public short getFillBackgroundColor()
     {
-        return format.getFillBackground();
+    	short result = format.getFillBackground();
+    	//JMH: Do this ridiculous conversion, and let HSSFCellStyle
+    	//internally migrate back and forth
+    	if (result == (HSSFColor.AUTOMATIC.index+1))
+    		return HSSFColor.AUTOMATIC.index;
+    	else return result;
     }
 
     /**
      * set the foreground fill color
+     * <i>Note: Ensure Foreground color is set prior to background color.</i>
      * @param bg  color
      */
-
     public void setFillForegroundColor(short bg)
     {
         format.setFillForeground(bg);
+        checkDefaultBackgroundFills();
     }
 
     /**
      * get the foreground fill color
+     * @see org.apache.poi.hssf.usermodel.HSSFPalette#getColor(short)
      * @return fill color
      */
-
     public short getFillForegroundColor()
     {
         return format.getFillForeground();
     }
-
 }
