@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -22,10 +23,16 @@
 package org.apache.poi.hssf.usermodel;
 
 import org.apache.poi.ddf.EscherRecord;
+import org.apache.poi.hssf.model.FormulaParser;
 import org.apache.poi.hssf.model.Sheet;
 import org.apache.poi.hssf.model.Workbook;
 import org.apache.poi.hssf.record.*;
+import org.apache.poi.hssf.record.formula.Ptg;
+import org.apache.poi.hssf.record.formula.ReferencePtg;
+import org.apache.poi.hssf.util.HSSFCellRangeAddress;
+import org.apache.poi.hssf.util.HSSFDataValidation;
 import org.apache.poi.hssf.util.Region;
+import org.apache.poi.hssf.util.PaneInformation;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
 
@@ -33,7 +40,16 @@ import java.io.PrintWriter;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Stack;
 import java.util.TreeMap;
+import java.text.AttributedString;
+import java.text.NumberFormat;
+import java.text.DecimalFormat;
+import java.awt.font.TextLayout;
+import java.awt.font.FontRenderContext;
+import java.awt.font.TextAttribute;
+
+import java.awt.geom.AffineTransform;
 
 /**
  * High level representation of a worksheet.
@@ -41,6 +57,8 @@ import java.util.TreeMap;
  * @author  Glen Stampoultzis (glens at apache.org)
  * @author  Libin Roman (romal at vistaportal.com)
  * @author  Shawn Laubach (slaubach at apache dot org) (Just a little)
+ * @author  Jean-Pierre Paris (jean-pierre.paris at m4x dot org) (Just a little, too)
+ * @author  Yegor Kozlov (yegor at apache.org) (Autosizing columns)
  */
 
 public class HSSFSheet
@@ -73,7 +91,8 @@ public class HSSFSheet
 
     private Sheet sheet;
     private TreeMap rows;
-    private Workbook book;
+    protected Workbook book;
+    protected HSSFWorkbook workbook;
     private int firstrow;
     private int lastrow;
     private static POILogger log = POILogFactory.getLogger(HSSFSheet.class);
@@ -82,36 +101,38 @@ public class HSSFSheet
      * Creates new HSSFSheet   - called by HSSFWorkbook to create a sheet from
      * scratch.  You should not be calling this from application code (its protected anyhow).
      *
-     * @param book - lowlevel Workbook object associated with the sheet.
+     * @param workbook - The HSSF Workbook object associated with the sheet.
      * @see org.apache.poi.hssf.usermodel.HSSFWorkbook#createSheet()
      */
 
-    protected HSSFSheet(Workbook book)
+    protected HSSFSheet(HSSFWorkbook workbook)
     {
         sheet = Sheet.createSheet();
         rows = new TreeMap();   // new ArrayList(INITIAL_CAPACITY);
-        this.book = book;
+        this.workbook = workbook;
+        this.book = workbook.getWorkbook();
     }
 
     /**
      * Creates an HSSFSheet representing the given Sheet object.  Should only be
      * called by HSSFWorkbook when reading in an exisiting file.
      *
-     * @param book - lowlevel Workbook object associated with the sheet.
+     * @param workbook - The HSSF Workbook object associated with the sheet.
      * @param sheet - lowlevel Sheet object this sheet will represent
      * @see org.apache.poi.hssf.usermodel.HSSFWorkbook#createSheet()
      */
 
-    protected HSSFSheet(Workbook book, Sheet sheet)
+    protected HSSFSheet(HSSFWorkbook workbook, Sheet sheet)
     {
         this.sheet = sheet;
         rows = new TreeMap();
-        this.book = book;
+        this.workbook = workbook;
+        this.book = workbook.getWorkbook();
         setPropertiesFromSheet(sheet);
     }
 
-    HSSFSheet cloneSheet(Workbook book) {
-      return new HSSFSheet(book, sheet.cloneSheet());
+    HSSFSheet cloneSheet(HSSFWorkbook workbook) {
+      return new HSSFSheet(workbook, sheet.cloneSheet());
     }
 
 
@@ -242,10 +263,12 @@ public class HSSFSheet
         int rownum = lastrow - 1;
         HSSFRow r = getRow(rownum);
 
-        while (r == null && rownum >= 0)
+        while (r == null && rownum > 0)
         {
             r = getRow(--rownum);
         }
+        if (r == null)
+          return -1;
         return rownum;
     }
 
@@ -336,6 +359,116 @@ public class HSSFSheet
     {
         return lastrow;
     }
+    
+    /**
+     * Creates a data validation object
+     * @param obj_validation The Data validation object settings
+     */
+    public void addValidationData(HSSFDataValidation obj_validation)
+    {
+       if ( obj_validation == null )
+       {
+         return;
+       }
+       DVALRecord dvalRec = (DVALRecord)sheet.findFirstRecordBySid( DVALRecord.sid );
+       int eofLoc = sheet.findFirstRecordLocBySid( EOFRecord.sid );
+       if ( dvalRec == null )
+       {
+          dvalRec = new DVALRecord();
+          sheet.getRecords().add( eofLoc, dvalRec );
+       }
+       int curr_dvRecNo = dvalRec.getDVRecNo();
+       dvalRec.setDVRecNo(curr_dvRecNo+1);
+
+       //create dv record
+       DVRecord dvRecord = new DVRecord();
+
+       //dv record's option flags
+       dvRecord.setDataType( obj_validation.getDataValidationType() );
+       dvRecord.setErrorStyle(obj_validation.getErrorStyle());
+       dvRecord.setEmptyCellAllowed(obj_validation.getEmptyCellAllowed());
+       dvRecord.setSurppresDropdownArrow(obj_validation.getSurppressDropDownArrow());
+       dvRecord.setShowPromptOnCellSelected(obj_validation.getShowPromptBox());
+       dvRecord.setShowErrorOnInvalidValue(obj_validation.getShowErrorBox());
+       dvRecord.setConditionOperator(obj_validation.getOperator());
+
+       //string fields
+       dvRecord.setStringField( DVRecord.STRING_PROMPT_TITLE,obj_validation.getPromptBoxTitle());
+       dvRecord.setStringField( DVRecord.STRING_PROMPT_TEXT, obj_validation.getPromptBoxText());
+       dvRecord.setStringField( DVRecord.STRING_ERROR_TITLE, obj_validation.getErrorBoxTitle());
+       dvRecord.setStringField( DVRecord.STRING_ERROR_TEXT, obj_validation.getErrorBoxText());
+
+       //formula fields ( size and data )
+       String str_formula = obj_validation.getFirstFormula();
+       FormulaParser fp = new FormulaParser(str_formula+";",book);
+       fp.parse();
+       Stack ptg_arr = new Stack();
+       Ptg[] ptg  = fp.getRPNPtg();
+       int size = 0;
+       for (int k = 0; k < ptg.length; k++)
+       {
+           if ( ptg[k] instanceof org.apache.poi.hssf.record.formula.AreaPtg )
+           {
+              //we should set ptgClass to Ptg.CLASS_REF and explicit formula string to false
+              ptg[k].setClass(Ptg.CLASS_REF);
+              obj_validation.setExplicitListFormula(false);
+           }
+           size += ptg[k].getSize();
+           ptg_arr.push(ptg[k]);
+       }
+       dvRecord.setFirstFormulaRPN(ptg_arr);
+       dvRecord.setFirstFormulaSize((short)size);
+
+       dvRecord.setListExplicitFormula(obj_validation.getExplicitListFormula());
+
+       if ( obj_validation.getSecondFormula() != null )
+       {
+         str_formula = obj_validation.getSecondFormula();
+         fp = new FormulaParser(str_formula+";",book);
+         fp.parse();
+         ptg_arr = new Stack();
+         ptg  = fp.getRPNPtg();
+         size = 0;
+         for (int k = 0; k < ptg.length; k++)
+         {
+             size += ptg[k].getSize();
+             ptg_arr.push(ptg[k]);
+         }
+         dvRecord.setSecFormulaRPN(ptg_arr);
+         dvRecord.setSecFormulaSize((short)size);
+       }
+
+       //dv records cell range field
+       HSSFCellRangeAddress cell_range = new HSSFCellRangeAddress();
+       cell_range.addADDRStructure(obj_validation.getFirstRow(), obj_validation.getFirstColumn(), obj_validation.getLastRow(), obj_validation.getLastColumn());
+       dvRecord.setCellRangeAddress(cell_range);
+
+       //add dv record
+       eofLoc = sheet.findFirstRecordLocBySid( EOFRecord.sid );
+       sheet.getRecords().add( eofLoc, dvRecord );
+    }
+
+    /**
+     * Get the visibility state for a given column.
+     * @param column - the column to get (0-based)
+     * @param hidden - the visiblity state of the column
+     */
+
+    public void setColumnHidden(short column, boolean hidden)
+    {
+        sheet.setColumnHidden(column, hidden);
+    }
+
+    /**
+     * Get the hidden state for a given column.
+     * @param column - the column to set (0-based)
+     * @return hidden - the visiblity state of the column
+     */
+
+    public boolean isColumnHidden(short column)
+    {
+        return sheet.isColumnHidden(column);
+    }
 
     /**
      * set the width (in units of 1/256th of a character width)
@@ -461,6 +594,26 @@ public class HSSFSheet
                 region.getColumnTo());
     }
 
+    /**
+     * Whether a record must be inserted or not at generation to indicate that 
+     * formula must be recalculated when workbook is opened.
+     * @param value true if an uncalced record must be inserted or not at generation
+     */
+    public void setForceFormulaRecalculation(boolean value)
+    {
+    	sheet.setUncalced(value);
+    }
+    /**
+     * Whether a record must be inserted or not at generation to indicate that 
+     * formula must be recalculated when workbook is opened.
+     * @return true if an uncalced record must be inserted or not at generation
+     */
+    public boolean getForceFormulaRecalculation()
+    {
+    	return sheet.getUncalced();
+    }
+
+    
     /**
      * determines whether the output is vertically centered on the page.
      * @param value true to vertically center, false otherwise.
@@ -838,17 +991,49 @@ public class HSSFSheet
 	 * @return true => protection enabled; false => protection disabled
 	 */
 	public boolean getProtect() {
-		return getSheet().getProtect().getProtect();
+		return getSheet().isProtected()[0];
+	}
+
+	/**
+	 * @return hashed password
+	 */
+	public short getPassword() {
+		return getSheet().getPassword().getPassword();
+	}
+
+	/**
+	 * Answer whether object protection is enabled or disabled
+	 * @return true => protection enabled; false => protection disabled
+	 */
+	public boolean getObjectProtect() {
+		return getSheet().isProtected()[1];
+	}
+
+	/**
+	 * Answer whether scenario protection is enabled or disabled
+	 * @return true => protection enabled; false => protection disabled
+	 */
+	public boolean getScenarioProtect() {
+		return getSheet().isProtected()[2];
 	}
 
 	/**
 	 * Sets the protection on enabled or disabled
 	 * @param protect true => protection enabled; false => protection disabled
+         * @deprecated use protectSheet(String, boolean, boolean)
 	 */
 	public void setProtect(boolean protect) {
 		getSheet().getProtect().setProtect(protect);
 	}
 
+        /**
+         * Sets the protection enabled as well as the password
+         * @param password to set for protection
+         */
+        public void protectSheet(String password) {
+                getSheet().protectSheet(password, true, true); //protect objs&scenarios(normal)
+        }
+
     /**
      * Sets the zoom magnication for the sheet.  The zoom is expressed as a
      * fraction.  For example to express a zoom of 75% use 3 for the numerator
@@ -869,6 +1054,37 @@ public class HSSFSheet
         sclRecord.setDenominator((short)denominator);
         getSheet().setSCLRecord(sclRecord);
     }
+    
+    /**
+     * The top row in the visible view when the sheet is 
+     * first viewed after opening it in a viewer 
+     * @return short indicating the rownum (0 based) of the top row
+     */
+    public short getTopRow() 
+    {
+    	return sheet.getTopRow();
+    }
+    
+    /**
+     * The left col in the visible view when the sheet is 
+     * first viewed after opening it in a viewer 
+     * @return short indicating the rownum (0 based) of the top row
+     */
+    public short getLeftCol() 
+    {
+    	return sheet.getLeftCol();
+    }
+    
+    /**
+     * Sets desktop window pane display area, when the 
+     * file is first opened in a viewer.
+     * @param toprow the top row to show in desktop window pane
+     * @param leftcol the left column to show in desktop window pane
+     */
+    public void showInPane(short toprow, short leftcol){
+        this.sheet.setTopRow((short)toprow);
+        this.sheet.setLeftCol((short)leftcol);
+        }
 
 	/**
 	 * Shifts the merged regions left or right depending on mode
@@ -1007,10 +1223,66 @@ public class HSSFSheet
                     row2Replace.createCellFromRecord( cellRecord );
                     sheet.addValueRecord( rowNum + n, cellRecord );
                 }
+
+                // move comments if exist (can exist even if cell is null)
+                HSSFComment comment = getCellComment(rowNum, col);
+                if (comment != null) {
+                   comment.setRow(rowNum + n);
+                }
             }
         }
         if ( endRow == lastrow || endRow + n > lastrow ) lastrow = Math.min( endRow + n, 65535 );
         if ( startRow == firstrow || startRow + n < firstrow ) firstrow = Math.max( startRow + n, 0 );
+        
+        // Update any formulas on this sheet that point to
+        //  rows which have been moved
+        updateFormulasAfterShift(startRow, endRow, n);
+    }
+    
+    /**
+     * Called by shiftRows to update formulas on this sheet
+     *  to point to the new location of moved rows
+     */
+    private void updateFormulasAfterShift(int startRow, int endRow, int n) {
+    	// Need to look at every cell on the sheet
+    	// Not just those that were moved
+        Iterator ri = rowIterator();
+        while(ri.hasNext()) {
+        	HSSFRow r = (HSSFRow)ri.next();
+        	Iterator ci = r.cellIterator();
+        	while(ci.hasNext()) {
+        		HSSFCell c = (HSSFCell)ci.next();
+        		if(c.getCellType() == HSSFCell.CELL_TYPE_FORMULA) {
+        			// Since it's a formula cell, process the
+        			//  formula string, and look to see if
+        			//  it contains any references
+        			FormulaParser fp = new FormulaParser(c.getCellFormula(), workbook.getWorkbook());
+        			fp.parse();
+        			
+        			// Look for references, and update if needed
+        			Ptg[] ptgs = fp.getRPNPtg();
+        			boolean changed = false;
+        			for(int i=0; i<ptgs.length; i++) {
+        				if(ptgs[i] instanceof ReferencePtg) {
+        					ReferencePtg rptg = (ReferencePtg)ptgs[i];
+        					if(startRow <= rptg.getRowAsInt() &&
+        							rptg.getRowAsInt() <= endRow) {
+        						// References a row that moved
+        						rptg.setRow(rptg.getRowAsInt() + n);
+        						changed = true;
+        					}
+        				}
+        			}
+        			// If any references were changed, then
+        			//  re-create the formula string
+        			if(changed) {
+        				c.setCellFormula(
+        						fp.toFormulaString(ptgs)
+        				);
+        			}
+        		}
+        	}
+        }
     }
 
     protected void insertChartRecords( List records )
@@ -1020,7 +1292,7 @@ public class HSSFSheet
     }
 
     /**
-     * Creates a split (freezepane).
+     * Creates a split (freezepane). Any existing freezepane or split pane is overwritten.
      * @param colSplit      Horizonatal position of split.
      * @param rowSplit      Vertical position of split.
      * @param topRow        Top row visible in bottom pane
@@ -1036,7 +1308,7 @@ public class HSSFSheet
     }
 
     /**
-     * Creates a split (freezepane).
+     * Creates a split (freezepane). Any existing freezepane or split pane is overwritten.
      * @param colSplit      Horizonatal position of split.
      * @param rowSplit      Vertical position of split.
      */
@@ -1046,7 +1318,7 @@ public class HSSFSheet
     }
 
     /**
-     * Creates a split pane.
+     * Creates a split pane. Any existing freezepane or split pane is overwritten.
      * @param xSplitPos      Horizonatal position of split (in 1/20th of a point).
      * @param ySplitPos      Vertical position of split (in 1/20th of a point).
      * @param topRow        Top row visible in bottom pane
@@ -1062,6 +1334,14 @@ public class HSSFSheet
     {
         getSheet().createSplitPane( xSplitPos, ySplitPos, topRow, leftmostColumn, activePane );
     }
+    
+    /**
+     * Returns the information regarding the currently configured pane (split or freeze).
+     * @return null if no pane configured, or the pane information.
+     */
+    public PaneInformation getPaneInformation() {
+      return getSheet().getPaneInformation();
+    }
 
     /**
      * Sets whether the gridlines are shown in a viewer.
@@ -1113,7 +1393,7 @@ public class HSSFSheet
     
     /**
      * Sets a page break at the indicated row
-     * @param row
+     * @param row FIXME: Document this!
      */
     public void setRowBreak(int row) {
     	validateRow(row);
@@ -1122,8 +1402,8 @@ public class HSSFSheet
 
     /**
      * Determines if there is a page break at the indicated row
-     * @param row
-     * @return
+     * @param row FIXME: Document this!
+     * @return FIXME: Document this!
      */
     public boolean isRowBroken(int row) {
     	return sheet.isRowBroken(row);
@@ -1139,34 +1419,42 @@ public class HSSFSheet
     
     /**
      * Retrieves all the horizontal page breaks
-     * @return
+     * @return all the horizontal page breaks, or null if there are no row page breaks
      */
     public int[] getRowBreaks(){
-    	//we can probably cache this information, but this should be a sparsely used function 
-    	int[] returnValue = new int[sheet.getNumRowBreaks()];
-    	Iterator iterator = sheet.getRowBreaks();
-    	int i = 0;
-    	while (iterator.hasNext()) {
+    	//we can probably cache this information, but this should be a sparsely used function
+    	int count = sheet.getNumRowBreaks();
+    	if (count > 0) {
+    	  int[] returnValue = new int[count];
+    	  Iterator iterator = sheet.getRowBreaks();
+    	  int i = 0;
+    	  while (iterator.hasNext()) {
     		PageBreakRecord.Break breakItem = (PageBreakRecord.Break)iterator.next();
     		returnValue[i++] = (int)breakItem.main;
+    	  }
+    	  return returnValue;
     	}
-    	return returnValue;
+    	return null;
     }
 
     /**
      * Retrieves all the vertical page breaks
-     * @return
+     * @return all the vertical page breaks, or null if there are no column page breaks
      */
     public short[] getColumnBreaks(){
     	//we can probably cache this information, but this should be a sparsely used function 
-    	short[] returnValue = new short[sheet.getNumColumnBreaks()];
-    	Iterator iterator = sheet.getColumnBreaks();
-    	int i = 0;
-    	while (iterator.hasNext()) {
+    	int count = sheet.getNumColumnBreaks();
+    	if (count > 0) {
+    	  short[] returnValue = new short[count];
+    	  Iterator iterator = sheet.getColumnBreaks();
+    	  int i = 0;
+    	  while (iterator.hasNext()) {
     		PageBreakRecord.Break breakItem = (PageBreakRecord.Break)iterator.next();
     		returnValue[i++] = breakItem.main;
+    	  }
+    	  return returnValue;
     	}
-    	return returnValue;
+    	return null;
     }
     
     
@@ -1181,8 +1469,8 @@ public class HSSFSheet
 
     /**
      * Determines if there is a page break at the indicated column
-     * @param column
-     * @return
+     * @param column FIXME: Document this!
+     * @return FIXME: Document this!
      */
     public boolean isColumnBroken(short column) {
     	return sheet.isColumnBroken(column);
@@ -1218,25 +1506,29 @@ public class HSSFSheet
      * Aggregates the drawing records and dumps the escher record hierarchy
      * to the standard output.
      */
-    public void dumpDrawingRecords()
+    public void dumpDrawingRecords(boolean fat)
     {
-        sheet.aggregateDrawingRecords(book.getDrawingManager());
+        sheet.aggregateDrawingRecords(book.getDrawingManager(), false);
 
         EscherAggregate r = (EscherAggregate) getSheet().findFirstRecordBySid(EscherAggregate.sid);
         List escherRecords = r.getEscherRecords();
+        PrintWriter w = new PrintWriter(System.out);
         for ( Iterator iterator = escherRecords.iterator(); iterator.hasNext(); )
         {
             EscherRecord escherRecord = (EscherRecord) iterator.next();
-            PrintWriter w = new PrintWriter(System.out);
-            escherRecord.display(w, 0);
-            w.close();
+            if (fat)
+                System.out.println(escherRecord.toString());
+            else
+                escherRecord.display(w, 0);
         }
+        w.flush();
     }
 
     /**
-     * Creates the toplevel drawing patriarch.  This will have the effect of
-     * removing any existing drawings on this sheet.
-     *
+     * Creates the top-level drawing patriarch.  This will have
+     *  the effect of removing any existing drawings on this
+     *  sheet.
+     * This may then be used to add graphics or charts
      * @return  The new patriarch.
      */
     public HSSFPatriarch createDrawingPatriarch()
@@ -1244,14 +1536,57 @@ public class HSSFSheet
         // Create the drawing group if it doesn't already exist.
         book.createDrawingGroup();
 
-        sheet.aggregateDrawingRecords(book.getDrawingManager());
+        sheet.aggregateDrawingRecords(book.getDrawingManager(), true);
         EscherAggregate agg = (EscherAggregate) sheet.findFirstRecordBySid(EscherAggregate.sid);
-        HSSFPatriarch patriarch = new HSSFPatriarch(this);
+        HSSFPatriarch patriarch = new HSSFPatriarch(this, agg);
         agg.clear();     // Initially the behaviour will be to clear out any existing shapes in the sheet when
                          // creating a new patriarch.
         agg.setPatriarch(patriarch);
         return patriarch;
     }
+    
+    /**
+     * Returns the top-level drawing patriach, if there is
+     *  one.
+     * This will hold any graphics or charts for the sheet.
+     * WARNING - calling this will trigger a parsing of the
+     *  associated escher records. Any that aren't supported
+     *  (such as charts and complex drawing types) will almost
+     *  certainly be lost or corrupted when written out. Only
+     *  use this with simple drawings, otherwise call
+     *  {@link HSSFSheet#createDrawingPatriarch()} and
+     *  start from scratch!
+     */
+    public HSSFPatriarch getDrawingPatriarch() {
+    	book.findDrawingGroup();
+    	
+    	// If there's now no drawing manager, then there's
+    	//  no drawing escher records on the workbook
+    	if(book.getDrawingManager() == null) {
+    		return null;
+    	}
+    	
+    	int found = sheet.aggregateDrawingRecords(
+    			book.getDrawingManager(), false
+    	);
+    	if(found == -1) {
+    		// Workbook has drawing stuff, but this sheet doesn't
+    		return null;
+    	}
+    	
+    	// Grab our aggregate record, and wire it up
+        EscherAggregate agg = (EscherAggregate) sheet.findFirstRecordBySid(EscherAggregate.sid);
+        HSSFPatriarch patriarch = new HSSFPatriarch(this, agg);
+        agg.setPatriarch(patriarch);
+        
+        // Have it process the records into high level objects
+        //  as best it can do (this step may eat anything
+        //  that isn't supported, you were warned...)
+        agg.convertRecordsToUserModel();
+        
+        // Return what we could cope with
+        return patriarch;
+    }
 
     /**
      * Expands or collapses a column group.
@@ -1295,5 +1630,190 @@ public class HSSFSheet
         sheet.setRowGroupCollapsed( row, collapse );
     }
 
+    /**
+     * Sets the default column style for a given column.  POI will only apply this style to new cells added to the sheet.
+     *
+     * @param column the column index
+     * @param style the style to set
+     */
+    public void setDefaultColumnStyle(short column, HSSFCellStyle style) {
+	sheet.setColumn(column, new Short(style.getIndex()), null, null, null, null);
+    }
+
+    /**
+     * Adjusts the column width to fit the contents.
+     *
+     * This process can be relatively slow on large sheets, so this should
+     *  normally only be called once per column, at the end of your
+     *  processing.
+     *
+     * @param column the column index
+     */
+    public void autoSizeColumn(short column) {
+        AttributedString str;
+        TextLayout layout;
+        /**
+         * Excel measures columns in units of 1/256th of a character width
+         * but the docs say nothing about what particular character is used.
+         * '0' looks to be a good choice.
+         */
+        char defaultChar = '0';
+       
+        /**
+         * This is the multiple that the font height is scaled by when determining the
+         * boundary of rotated text.
+         */
+        double fontHeightMultiple = 2.0;
+       
+        FontRenderContext frc = new FontRenderContext(null, true, true);
+
+        HSSFWorkbook wb = new HSSFWorkbook(book);
+        HSSFFont defaultFont = wb.getFontAt((short) 0);
+
+        str = new AttributedString("" + defaultChar);
+        copyAttributes(defaultFont, str, 0, 1);
+        layout = new TextLayout(str.getIterator(), frc);
+        int defaultCharWidth = (int)layout.getAdvance();
+
+        double width = -1;
+        for (Iterator it = rowIterator(); it.hasNext();) {
+            HSSFRow row = (HSSFRow) it.next();
+            HSSFCell cell = row.getCell(column);
+
+            boolean isCellInMergedRegion = false;
+            for (int i = 0 ; i < getNumMergedRegions() && ! isCellInMergedRegion; i++) {
+                isCellInMergedRegion = getMergedRegionAt(i).contains(row.getRowNum(), column);
+            }
+
+            if (cell == null | isCellInMergedRegion) continue;
+
+            HSSFCellStyle style = cell.getCellStyle();
+            HSSFFont font = wb.getFontAt(style.getFontIndex());
+
+            if (cell.getCellType() == HSSFCell.CELL_TYPE_STRING) {
+                HSSFRichTextString rt = cell.getRichStringCellValue();
+                String[] lines = rt.getString().split("\\n");
+                for (int i = 0; i < lines.length; i++) {
+                    String txt = lines[i] + defaultChar;
+                    str = new AttributedString(txt);
+                    copyAttributes(font, str, 0, txt.length());
+
+                    if (rt.numFormattingRuns() > 0) {
+                        for (int j = 0; j < lines[i].length(); j++) {
+                            int idx = rt.getFontAtIndex(j);
+                            if (idx != 0) {
+                                HSSFFont fnt = wb.getFontAt((short) idx);
+                                copyAttributes(fnt, str, j, j + 1);
+                            }
+                        }
+                    }
+
+                    layout = new TextLayout(str.getIterator(), frc);
+                    if(style.getRotation() != 0){
+                        /*
+                         * Transform the text using a scale so that it's height is increased by a multiple of the leading,
+                         * and then rotate the text before computing the bounds. The scale results in some whitespace around
+                         * the unrotated top and bottom of the text that normally wouldn't be present if unscaled, but
+                         * is added by the standard Excel autosize.
+                         */
+                        AffineTransform trans = new AffineTransform();
+                        trans.concatenate(AffineTransform.getRotateInstance(style.getRotation()*2.0*Math.PI/360.0));
+                        trans.concatenate(
+                        AffineTransform.getScaleInstance(1, fontHeightMultiple)
+                        );
+                        width = Math.max(width, layout.getOutline(trans).getBounds().getWidth() / defaultCharWidth);
+                    } else {
+                        width = Math.max(width, layout.getBounds().getWidth() / defaultCharWidth);
+                    }
+                }
+            } else {
+                String sval = null;
+                if (cell.getCellType() == HSSFCell.CELL_TYPE_NUMERIC) {
+                    HSSFDataFormat dataformat = wb.createDataFormat();
+                    short idx = style.getDataFormat();
+                    String format = dataformat.getFormat(idx).replaceAll("\"", "");
+                    double value = cell.getNumericCellValue();
+                    try {
+                        NumberFormat fmt;
+                        if ("General".equals(format))
+                            sval = "" + value;
+                        else
+                        {
+                            fmt = new DecimalFormat(format);
+                            sval = fmt.format(value);
+                        }
+                    } catch (Exception e) {
+                        sval = "" + value;
+                    }
+                } else if (cell.getCellType() == HSSFCell.CELL_TYPE_BOOLEAN) {
+                    sval = String.valueOf(cell.getBooleanCellValue());
+                }
+                if(sval != null) {
+                    String txt = sval + defaultChar;
+                    str = new AttributedString(txt);
+                    copyAttributes(font, str, 0, txt.length());
+
+                    layout = new TextLayout(str.getIterator(), frc);
+                    if(style.getRotation() != 0){
+                        /*
+                         * Transform the text using a scale so that it's height is increased by a multiple of the leading,
+                         * and then rotate the text before computing the bounds. The scale results in some whitespace around
+                         * the unrotated top and bottom of the text that normally wouldn't be present if unscaled, but
+                         * is added by the standard Excel autosize.
+                         */
+                        AffineTransform trans = new AffineTransform();
+                        trans.concatenate(AffineTransform.getRotateInstance(style.getRotation()*2.0*Math.PI/360.0));
+                        trans.concatenate(
+                        AffineTransform.getScaleInstance(1, fontHeightMultiple)
+                        );
+                        width = Math.max(width, layout.getOutline(trans).getBounds().getWidth() / defaultCharWidth);
+                    } else {
+                        width = Math.max(width, layout.getBounds().getWidth() / defaultCharWidth);
+                    }
+                }
+            }
+
+            if (width != -1) {
+                if (width > Short.MAX_VALUE) { //width can be bigger that Short.MAX_VALUE!
+                     width = Short.MAX_VALUE;
+                }
+                sheet.setColumnWidth(column, (short) (width * 256));
+            }
+        }
+    }
+
+    /**
+     * Copy text attributes from the supplied HSSFFont to Java2D AttributedString
+     */
+    private void copyAttributes(HSSFFont font, AttributedString str, int startIdx, int endIdx){
+        str.addAttribute(TextAttribute.FAMILY, font.getFontName(), startIdx, endIdx);
+        str.addAttribute(TextAttribute.SIZE, new Float(font.getFontHeightInPoints()));
+        if (font.getBoldweight() == HSSFFont.BOLDWEIGHT_BOLD) str.addAttribute(TextAttribute.WEIGHT, TextAttribute.WEIGHT_BOLD, startIdx, endIdx);
+        if (font.getItalic() ) str.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, startIdx, endIdx);
+        if (font.getUnderline() == HSSFFont.U_SINGLE ) str.addAttribute(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON, startIdx, endIdx);
+    }
+
+    /**
+     * Returns cell comment for the specified row and column
+     *
+     * @return cell comment or <code>null</code> if not found
+     */
+     public HSSFComment getCellComment(int row, int column){
+        // Don't call findCellComment directly, otherwise
+        //  two calls to this method will result in two
+        //  new HSSFComment instances, which is bad
+        HSSFRow r = getRow(row);
+        if(r != null) {
+            HSSFCell c = r.getCell((short)column);
+            if(c != null) {
+                return c.getCellComment();
+            } else {
+                // No cell, so you will get new
+                //  objects every time, sorry...
+                return HSSFCell.findCellComment(sheet, row, column);
+            }
+        }
+        return null;
+    }
 
 }
