@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -22,15 +23,11 @@
  */
 package org.apache.poi.hssf.usermodel;
 
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Stack;
-
+import org.apache.poi.POIDocument;
+import org.apache.poi.ddf.EscherBSERecord;
+import org.apache.poi.ddf.EscherBitmapBlip;
+import org.apache.poi.ddf.EscherRecord;
+import org.apache.poi.ddf.EscherBlipRecord;
 import org.apache.poi.hssf.eventmodel.EventRecordFactory;
 import org.apache.poi.hssf.model.Sheet;
 import org.apache.poi.hssf.model.Workbook;
@@ -39,14 +36,21 @@ import org.apache.poi.hssf.record.formula.Area3DPtg;
 import org.apache.poi.hssf.record.formula.MemFuncPtg;
 import org.apache.poi.hssf.record.formula.UnionPtg;
 import org.apache.poi.hssf.util.CellReference;
-import org.apache.poi.poifs.filesystem.DirectoryEntry;
-import org.apache.poi.poifs.filesystem.DocumentEntry;
-import org.apache.poi.poifs.filesystem.DocumentInputStream;
-import org.apache.poi.poifs.filesystem.Entry;
-import org.apache.poi.poifs.filesystem.POIFSFileSystem;
+import org.apache.poi.poifs.filesystem.*;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
 
+import java.io.ByteArrayInputStream;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.io.PrintWriter;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Stack;
+
 /**
  * High level representation of a workbook.  This is the first object most users
  * will construct whether they are reading or writing a workbook.  It is also the
@@ -60,8 +64,7 @@ import org.apache.poi.util.POILogger;
  * @version 2.0-pre
  */
 
-public class HSSFWorkbook
-        extends java.lang.Object
+public class HSSFWorkbook extends POIDocument
 {
     private static final int DEBUG = POILogger.DEBUG;
 
@@ -84,7 +87,7 @@ public class HSSFWorkbook
      * this holds the HSSFSheet objects attached to this workbook
      */
 
-    private ArrayList sheets;
+    protected ArrayList sheets;
 
     /**
      * this holds the HSSFName objects attached to this workbook
@@ -98,12 +101,6 @@ public class HSSFWorkbook
      */
     private boolean   preserveNodes;
 
-    /**
-     * if you do preserve the nodes, you'll need to hold the whole POIFS in
-     * memory.
-     */
-    private POIFSFileSystem poifs;
-
     /**
      * Used to keep track of the data formatter so that all
      * createDataFormatter calls return the same one for a given
@@ -112,18 +109,39 @@ public class HSSFWorkbook
      */
     private HSSFDataFormat formatter;
 
+
+    /** Extended windows meta file */
+    public static final int PICTURE_TYPE_EMF = 2;
+    /** Windows Meta File */
+    public static final int PICTURE_TYPE_WMF = 3;
+    /** Mac PICT format */
+    public static final int PICTURE_TYPE_PICT = 4;
+    /** JPEG format */
+    public static final int PICTURE_TYPE_JPEG = 5;
+    /** PNG format */
+    public static final int PICTURE_TYPE_PNG = 6;
+    /** Device independant bitmap */
+    public static final int PICTURE_TYPE_DIB = 7;
+
+
     private static POILogger log = POILogFactory.getLogger(HSSFWorkbook.class);
 
+
+
     /**
      * Creates new HSSFWorkbook from scratch (start here!)
      *
      */
-
     public HSSFWorkbook()
     {
-        workbook = Workbook.createWorkbook();
-        sheets = new ArrayList(INITIAL_CAPACITY);
-        names  = new ArrayList(INITIAL_CAPACITY);
+        this(Workbook.createWorkbook());
+    }
+
+    protected HSSFWorkbook( Workbook book )
+    {
+        workbook = book;
+        sheets = new ArrayList( INITIAL_CAPACITY );
+        names = new ArrayList( INITIAL_CAPACITY );
     }
 
     public HSSFWorkbook(POIFSFileSystem fs) throws IOException {
@@ -137,7 +155,8 @@ public class HSSFWorkbook
      * @param fs the POI filesystem that contains the Workbook stream.
      * @param preserveNodes whether to preseve other nodes, such as
      *        macros.  This takes more memory, so only say yes if you
-     *        need to.
+     *        need to. If set, will store all of the POIFSFileSystem
+     *        in memory
      * @see org.apache.poi.poifs.filesystem.POIFSFileSystem
      * @exception IOException if the stream cannot be read
      */
@@ -147,19 +166,44 @@ public class HSSFWorkbook
     {
         this.preserveNodes = preserveNodes;
 
-        if (preserveNodes) {
-           this.poifs = fs;
+        // Read in the HPSF properties
+        this.filesystem = fs;
+        readProperties();
+        
+        // If we're not preserving nodes, don't track the
+        //  POIFS any more
+        if(! preserveNodes) {
+           this.filesystem = null;
         }
 
         sheets = new ArrayList(INITIAL_CAPACITY);
         names  = new ArrayList(INITIAL_CAPACITY);
+        
+        // Normally, the Workbook will be in a POIFS Stream
+        //  called "Workbook". However, some wierd XLS generators
+        //  put theirs in one called "WORKBOOK"
+        String workbookName = "Workbook";
+        try {
+        	fs.getRoot().getEntry(workbookName);
+        	// Is the default name
+        } catch(FileNotFoundException fe) {
+        	// Try the upper case form
+        	try {
+        		workbookName = "WORKBOOK";
+        		fs.getRoot().getEntry(workbookName);
+        	} catch(FileNotFoundException wfe) {
+        		// Doesn't contain it in either form
+        		throw new IllegalArgumentException("The supplied POIFSFileSystem contained neither a 'Workbook' entry, nor a 'WORKBOOK' entry. Is it really an excel file?");
+        	}
+        }
 
-        InputStream stream = fs.createDocumentInputStream("Workbook");
+        
+        // Grab the data from the workbook stream, however
+        //  it happens to be spelt.
+        InputStream stream = fs.createDocumentInputStream(workbookName);
 
         EventRecordFactory factory = new EventRecordFactory();
 
-
-
         List records = RecordFactory.createRecords(stream);
 
         workbook = Workbook.createWorkbook(records);
@@ -167,14 +211,19 @@ public class HSSFWorkbook
         int recOffset = workbook.getNumRecords();
         int sheetNum = 0;
 
+        // convert all LabelRecord records to LabelSSTRecord
+        convertLabelRecords(records, recOffset);        
         while (recOffset < records.size())
         {
             Sheet sheet = Sheet.createSheet(records, sheetNum++, recOffset );
 
             recOffset = sheet.getEofLoc()+1;
-            sheet.convertLabelRecords(
-                    workbook);   // convert all LabelRecord records to LabelSSTRecord
-            HSSFSheet hsheet = new HSSFSheet(workbook, sheet);
+            if (recOffset == 1)
+            {
+                break;
+            }
+
+            HSSFSheet hsheet = new HSSFSheet(this, sheet);
 
             sheets.add(hsheet);
 
@@ -220,6 +269,51 @@ public class HSSFWorkbook
 
         // none currently
     }
+    
+    /**
+      * This is basically a kludge to deal with the now obsolete Label records.  If
+      * you have to read in a sheet that contains Label records, be aware that the rest
+      * of the API doesn't deal with them, the low level structure only provides read-only
+      * semi-immutable structures (the sets are there for interface conformance with NO
+      * impelmentation).  In short, you need to call this function passing it a reference
+      * to the Workbook object.  All labels will be converted to LabelSST records and their
+      * contained strings will be written to the Shared String tabel (SSTRecord) within
+      * the Workbook.
+      *
+      * @param wb sheet's matching low level Workbook structure containing the SSTRecord.
+      * @see org.apache.poi.hssf.record.LabelRecord
+      * @see org.apache.poi.hssf.record.LabelSSTRecord
+      * @see org.apache.poi.hssf.record.SSTRecord
+      */
+ 
+     private void convertLabelRecords(List records, int offset)
+     {
+         if (log.check( POILogger.DEBUG ))
+             log.log(POILogger.DEBUG, "convertLabelRecords called");
+         for (int k = offset; k < records.size(); k++)
+         {
+             Record rec = ( Record ) records.get(k);
+
+             if (rec.getSid() == LabelRecord.sid)
+             {
+                 LabelRecord oldrec = ( LabelRecord ) rec;
+
+                 records.remove(k);
+                 LabelSSTRecord newrec   = new LabelSSTRecord();
+                 int            stringid =
+                     workbook.addSSTString(new UnicodeString(oldrec.getValue()));
+
+                 newrec.setRow(oldrec.getRow());
+                 newrec.setColumn(oldrec.getColumn());
+                 newrec.setXFIndex(oldrec.getXFIndex());
+                 newrec.setSSTIndex(stringid);
+                       records.add(k, newrec);
+             }
+         }
+         if (log.check( POILogger.DEBUG ))
+             log.log(POILogger.DEBUG, "convertLabelRecords exit");
+     }
+    
 
     /**
      * sets the order of appearance for a given sheet.
@@ -229,10 +323,59 @@ public class HSSFWorkbook
      */
 
     public void setSheetOrder(String sheetname, int pos ) {
+        sheets.add(pos,sheets.remove(getSheetIndex(sheetname)));
         workbook.setSheetOrder(sheetname, pos);
     }
+    
+    /**
+     * sets the tab whose data is actually seen when the sheet is opened.
+     * This may be different from the "selected sheet" since excel seems to
+     * allow you to show the data of one sheet when another is seen "selected"
+     * in the tabs (at the bottom).
+     * @see org.apache.poi.hssf.usermodel.HSSFSheet#setSelected(boolean)
+     * @param index
+     */
+    public void setSelectedTab(short index) {
+        workbook.getWindowOne().setSelectedTab(index);
+    }
+    
+    /**
+     * gets the tab whose data is actually seen when the sheet is opened.
+     * This may be different from the "selected sheet" since excel seems to
+     * allow you to show the data of one sheet when another is seen "selected"
+     * in the tabs (at the bottom).
+     * @see org.apache.poi.hssf.usermodel.HSSFSheet#setSelected(boolean)
+     */
+    public short getSelectedTab() {
+        return workbook.getWindowOne().getSelectedTab();
+    }
+    
+    /**
+     * sets the first tab that is displayed in the list of tabs
+     * in excel.
+     * @param index
+     */
+    public void setDisplayedTab(short index) {
+        workbook.getWindowOne().setDisplayedTab(index);
+    }
+    
+    /**
+     * sets the first tab that is displayed in the list of tabs
+     * in excel.
+     */
+    public short getDisplayedTab() {
+        return workbook.getWindowOne().getDisplayedTab();
+    }
 
+    /**
+     * @deprecated POI will now properly handle unicode strings without
+     * forceing an encoding
+     */
     public final static byte ENCODING_COMPRESSED_UNICODE = 0;
+    /**
+     * @deprecated POI will now properly handle unicode strings without
+     * forceing an encoding
+     */
     public final static byte ENCODING_UTF_16             = 1;
 
 
@@ -241,16 +384,34 @@ public class HSSFWorkbook
      * Will throw IllegalArgumentException if the name is greater than 31 chars
      * or contains /\?*[]
      * @param sheet number (0 based)
-     * @param sheet name
      */
-
     public void setSheetName(int sheet, String name)
     {
-        workbook.setSheetName( sheet, name, ENCODING_COMPRESSED_UNICODE );
+        if (workbook.doesContainsSheetName( name, sheet ))
+            throw new IllegalArgumentException( "The workbook already contains a sheet with this name" );
+
+        if (sheet > (sheets.size() - 1))
+        {
+            throw new RuntimeException("Sheet out of bounds");
+        }
+        
+        workbook.setSheetName( sheet, name);
     }
 
+    
+    /**
+     * set the sheet name forcing the encoding. Forcing the encoding IS A BAD IDEA!!!
+     * @deprecated 3-Jan-2006 POI now automatically detects unicode and sets the encoding
+     * appropriately. Simply use setSheetName(int sheet, String encoding) 
+     * @throws IllegalArgumentException if the name is greater than 31 chars
+     * or contains /\?*[]
+     * @param sheet number (0 based)
+     */    
     public void setSheetName( int sheet, String name, short encoding )
     {
+        if (workbook.doesContainsSheetName( name, sheet ))
+            throw new IllegalArgumentException( "The workbook already contains a sheet with this name" );
+
         if (sheet > (sheets.size() - 1))
         {
             throw new RuntimeException("Sheet out of bounds");
@@ -284,6 +445,35 @@ public class HSSFWorkbook
         return workbook.getSheetName(sheet);
     }
 
+    /**
+     * check whether a sheet is hidden
+     * @param sheet Number
+     * @return True if sheet is hidden
+     */
+
+    public boolean isSheetHidden(int sheet) {
+        if (sheet > (sheets.size() - 1))
+        {
+            throw new RuntimeException("Sheet out of bounds");
+        }
+        return workbook.isSheetHidden(sheet);
+    }
+
+    /**
+     * Hide or unhide a sheet
+     * 
+     * @param sheetnum The sheet number
+     * @param hidden True to mark the sheet as hidden, false otherwise
+     */
+
+    public void setSheetHidden(int sheet, boolean hidden) {
+        if (sheet > (sheets.size() - 1))
+        {
+            throw new RuntimeException("Sheet out of bounds");
+        }
+        workbook.setSheetHidden(sheet,hidden);
+    }
+    
     /*
      * get the sheet's index
      * @param name  sheet name
@@ -301,6 +491,20 @@ public class HSSFWorkbook
         return retval;
     }
 
+    /** Returns the index of the given sheet
+     * @param sheet the sheet to look up
+     * @return index of the sheet (0 based)
+     */
+    public int getSheetIndex(HSSFSheet sheet)
+    {
+    	for(int i=0; i<sheets.size(); i++) {
+    		if(sheets.get(i) == sheet) {
+    			return i;
+    		}
+    	}
+    	return -1;
+    }
+
     /**
      * create an HSSFSheet for this HSSFWorkbook, adds it to the sheets and returns
      * the high level representation.  Use this to create new sheets.
@@ -313,7 +517,7 @@ public class HSSFWorkbook
 
 //        if (getNumberOfSheets() == 3)
 //            throw new RuntimeException("You cannot have more than three sheets in HSSF 1.0");
-        HSSFSheet sheet = new HSSFSheet(workbook);
+        HSSFSheet sheet = new HSSFSheet(this);
 
         sheets.add(sheet);
         workbook.setSheetName(sheets.size() - 1,
@@ -334,16 +538,26 @@ public class HSSFWorkbook
       HSSFSheet srcSheet = (HSSFSheet)sheets.get(sheetNum);
       String srcName = workbook.getSheetName(sheetNum);
       if (srcSheet != null) {
-        HSSFSheet clonedSheet = srcSheet.cloneSheet(workbook);
+        HSSFSheet clonedSheet = srcSheet.cloneSheet(this);
         WindowTwoRecord windowTwo = (WindowTwoRecord) clonedSheet.getSheet().findFirstRecordBySid(WindowTwoRecord.sid);
         windowTwo.setSelected(sheets.size() == 1);
         windowTwo.setPaged(sheets.size() == 1);
 
         sheets.add(clonedSheet);
-        if (srcName.length()<28) {
-            workbook.setSheetName(sheets.size()-1, srcName+"(2)");
-        }else {
-            workbook.setSheetName(sheets.size()-1,srcName.substring(0,28)+"(2)");
+        int i=1;
+        while (true) {
+        	//Try and find the next sheet name that is unique
+        	String name = srcName;
+        	String index = Integer.toString(i++);
+        	if (name.length()+index.length()+2<31)
+        	  name = name + "("+index+")";
+        	else name = name.substring(0, 31-index.length()-2)+"("+index+")";
+        	
+        	//If the sheet name is unique, then set it otherwise move on to the next number.
+        	if (workbook.getSheetIndex(name) == -1) {
+              workbook.setSheetName(sheets.size()-1, name);
+              break;
+        	}
         }
         return clonedSheet;
       }
@@ -360,10 +574,10 @@ public class HSSFWorkbook
 
     public HSSFSheet createSheet(String sheetname)
     {
+        if (workbook.doesContainsSheetName( sheetname, sheets.size() ))
+            throw new IllegalArgumentException( "The workbook already contains a sheet of this name" );
 
-//        if (getNumberOfSheets() == 3)
-//            throw new RuntimeException("You cannot have more than three sheets in HSSF 1.0");
-        HSSFSheet sheet = new HSSFSheet(workbook);
+        HSSFSheet sheet = new HSSFSheet(this);
 
         sheets.add(sheet);
         workbook.setSheetName(sheets.size() - 1, sheetname);
@@ -584,11 +798,14 @@ public class HSSFWorkbook
         NameRecord r = null;
         while ((r = (NameRecord) workbook.findNextRecordBySid(NameRecord.sid, index)) != null)
         {
-            int nameRecordSheetIndex = workbook.getSheetIndexFromExternSheetIndex(r.getEqualsToIndexToSheet() - 1);
-            if (isRowColHeaderRecord( r ) && nameRecordSheetIndex == sheetIndex)
-            {
-                return index;
-            }
+            int indexToSheet = r.getEqualsToIndexToSheet() -1;
+            if(indexToSheet > -1) { //ignore "GLOBAL" name records
+                int nameRecordSheetIndex = workbook.getSheetIndexFromExternSheetIndex(indexToSheet);
+                if (isRowColHeaderRecord( r ) && nameRecordSheetIndex == sheetIndex)
+                {
+                    return index;
+                }
+            } 
             index++;
         }
 
@@ -614,6 +831,9 @@ public class HSSFWorkbook
         {
             fontindex++;   // THERE IS NO FOUR!!
         }
+        if(fontindex == Short.MAX_VALUE){
+            throw new IllegalArgumentException("Maximum number of fonts was exceeded");
+        }
         HSSFFont retval = new HSSFFont(fontindex, font);
 
         return retval;
@@ -732,12 +952,25 @@ public class HSSFWorkbook
         byte[] bytes = getBytes();
         POIFSFileSystem fs = new POIFSFileSystem();
 
+        // For tracking what we've written out, used if we're
+        //  going to be preserving nodes
+        List excepts = new ArrayList(1);
+        
+        // Write out the Workbook stream
         fs.createDocument(new ByteArrayInputStream(bytes), "Workbook");
+        
+        // Write out our HPFS properties, if we have them
+        writeProperties(fs, excepts);
 
         if (preserveNodes) {
-            List excepts = new ArrayList(1);
+			// Don't write out the old Workbook, we'll be doing our new one
             excepts.add("Workbook");
-            copyNodes(this.poifs,fs,excepts);
+			// If the file had WORKBOOK instead of Workbook, we'll write it
+			//  out correctly shortly, so don't include the old one
+            excepts.add("WORKBOOK");
+
+			// Copy over all the other nodes to our new poifs
+            copyNodes(this.filesystem,fs,excepts);
         }
         fs.writeFilesystem(stream);
         //poifs.writeFilesystem(stream);
@@ -791,8 +1024,9 @@ public class HSSFWorkbook
 
             // byte[] sb = (byte[])sheetbytes.get(k);
             // System.arraycopy(sb, 0, retval, pos, sb.length);
-            pos += ((HSSFSheet) sheets.get(k)).getSheet().serialize(pos,
-                    retval);   // sb.length;
+            int len = ((HSSFSheet) sheets.get(k)).getSheet().serialize(pos,
+                                retval);
+            pos += len;   // sb.length;
         }
 /*        for (int k = pos; k < totalsize; k++)
         {
@@ -801,17 +1035,23 @@ public class HSSFWorkbook
         return retval;
     }
 
+    /** @deprecated Do not call this method from your applications. Use the methods
+     *  available in the HSSFRow to add string HSSFCells
+     */
     public int addSSTString(String string)
     {
-        return workbook.addSSTString(string);
+        return workbook.addSSTString(new UnicodeString(string));
     }
 
+    /** @deprecated Do not call this method from your applications. Use the methods
+     *  available in the HSSFRow to get string HSSFCells
+     */
     public String getSSTString(int index)
     {
-        return workbook.getSSTString(index);
+        return workbook.getSSTString(index).getString();
     }
 
-    Workbook getWorkbook()
+    protected Workbook getWorkbook()
     {
         return workbook;
     }
@@ -869,7 +1109,7 @@ public class HSSFWorkbook
 
 	/**
 	 * For the Convenience of Java Programmers maintaining pointers.
-	 * @see setPrintArea(int, String)
+	 * @see #setPrintArea(int, String)
 	 * @param sheetIndex Zero-based sheet index (0 = First Sheet)
 	 * @param startColumn Column to begin printarea
 	 * @param endColumn Column to end the printarea
@@ -926,6 +1166,9 @@ public class HSSFWorkbook
     }
 
     /** gets the named range index by his name
+     * <i>Note:</i>Excel named ranges are case-insensitive and
+     * this method performs a case-insensitive search.
+     * 
      * @param name named range name
      * @return named range index
      */
@@ -937,7 +1180,7 @@ public class HSSFWorkbook
         {
             String nameName = getNameName(k);
 
-            if (nameName.equals(name))
+            if (nameName.equalsIgnoreCase(name))
             {
                 retval = k;
                 break;
@@ -982,58 +1225,6 @@ public class HSSFWorkbook
         return new HSSFPalette(workbook.getCustomPalette());
     }
 
-   /**
-    * Copies nodes from one POIFS to the other minus the excepts
-    * @param source is the source POIFS to copy from
-    * @param target is the target POIFS to copy to
-    * @param excepts is a list of Strings specifying what nodes NOT to copy
-    */
-   private void copyNodes(POIFSFileSystem source, POIFSFileSystem target,
-                          List excepts) throws IOException {
-      //System.err.println("CopyNodes called");
-
-      DirectoryEntry root = source.getRoot();
-      DirectoryEntry newRoot = target.getRoot();
-
-      Iterator entries = root.getEntries();
-
-      while (entries.hasNext()) {
-         Entry entry = (Entry)entries.next();
-         if (!isInList(entry.getName(), excepts)) {
-             copyNodeRecursively(entry,newRoot);
-         }
-      }
-   }
-
-   private boolean isInList(String entry, List list) {
-       for (int k = 0; k < list.size(); k++) {
-          if (list.get(k).equals(entry)) {
-            return true;
-          }
-       }
-       return false;
-   }
-
-   private void copyNodeRecursively(Entry entry, DirectoryEntry target)
-   throws IOException {
-       //System.err.println("copyNodeRecursively called with "+entry.getName()+
-       //                   ","+target.getName());
-       DirectoryEntry newTarget = null;
-       if (entry.isDirectoryEntry()) {
-           newTarget = target.createDirectory(entry.getName());
-           Iterator entries = ((DirectoryEntry)entry).getEntries();
-
-           while (entries.hasNext()) {
-              copyNodeRecursively((Entry)entries.next(),newTarget);
-           }
-       } else {
-         DocumentEntry dentry = (DocumentEntry)entry;
-         DocumentInputStream dstream = new DocumentInputStream(dentry);
-         target.createDocument(dentry.getName(),dstream);
-         dstream.close();
-       }
-   }
-
     /** Test only. Do not use */
     public void insertChartRecord()
     {
@@ -1058,10 +1249,202 @@ public class HSSFWorkbook
            (byte)0x00, (byte)0x08, (byte)0x17, (byte)0x00, (byte)0x00,
            (byte)0x08, (byte)0xF7, (byte)0x00, (byte)0x00, (byte)0x10,
         };
-        UnknownRecord r = new UnknownRecord((short)0x00EB,(short)0x005a, data);
+        UnknownRecord r = new UnknownRecord((short)0x00EB, data);
         workbook.getRecords().add(loc, r);
     }
 
+    /**
+     * Spits out a list of all the drawing records in the workbook.
+     */
+    public void dumpDrawingGroupRecords(boolean fat)
+    {
+        DrawingGroupRecord r = (DrawingGroupRecord) workbook.findFirstRecordBySid( DrawingGroupRecord.sid );
+        r.decode();
+        List escherRecords = r.getEscherRecords();
+        PrintWriter w = new PrintWriter(System.out);
+        for ( Iterator iterator = escherRecords.iterator(); iterator.hasNext(); )
+        {
+            EscherRecord escherRecord = (EscherRecord) iterator.next();
+            if (fat)
+                System.out.println(escherRecord.toString());
+            else
+                escherRecord.display(w, 0);
+        }
+        w.flush();
+    }
 
+    /**
+     * Adds a picture to the workbook.
+     *
+     * @param pictureData       The bytes of the picture
+     * @param format            The format of the picture.  One of <code>PICTURE_TYPE_*</code>
+     *
+     * @return the index to this picture (1 based).
+     */
+    public int addPicture(byte[] pictureData, int format)
+    {
+        byte[] uid = newUID();
+        EscherBitmapBlip blipRecord = new EscherBitmapBlip();
+        blipRecord.setRecordId( (short) ( EscherBitmapBlip.RECORD_ID_START + format ) );
+        switch (format)
+        {
+            case PICTURE_TYPE_EMF:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_EMF);
+                break;
+            case PICTURE_TYPE_WMF:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_WMF);
+                break;
+            case PICTURE_TYPE_PICT:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_PICT);
+                break;
+            case PICTURE_TYPE_PNG:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_PNG);
+                break;
+            case HSSFWorkbook.PICTURE_TYPE_JPEG:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_JPEG);
+                break;
+            case HSSFWorkbook.PICTURE_TYPE_DIB:
+                blipRecord.setOptions(HSSFPictureData.MSOBI_DIB);
+                break;
+        }
 
+        blipRecord.setUID( uid );
+        blipRecord.setMarker( (byte) 0xFF );
+        blipRecord.setPictureData( pictureData );
+
+        EscherBSERecord r = new EscherBSERecord();
+        r.setRecordId( EscherBSERecord.RECORD_ID );
+        r.setOptions( (short) ( 0x0002 | ( format << 4 ) ) );
+        r.setBlipTypeMacOS( (byte) format );
+        r.setBlipTypeWin32( (byte) format );
+        r.setUid( uid );
+        r.setTag( (short) 0xFF );
+        r.setSize( pictureData.length + 25 );
+        r.setRef( 1 );
+        r.setOffset( 0 );
+        r.setBlipRecord( blipRecord );
+
+        return workbook.addBSERecord( r );
+    }
+
+    /**
+     * Gets all pictures from the Workbook.
+     *
+     * @return the list of pictures (a list of {@link HSSFPictureData} objects.)
+     */
+    public List getAllPictures()
+    {
+        // The drawing group record always exists at the top level, so we won't need to do this recursively.
+        List pictures = new ArrayList();
+        Iterator recordIter = workbook.getRecords().iterator();
+        while (recordIter.hasNext())
+        {
+            Object obj = recordIter.next();
+            if (obj instanceof AbstractEscherHolderRecord)
+            {
+                ((AbstractEscherHolderRecord) obj).decode();
+                List escherRecords = ((AbstractEscherHolderRecord) obj).getEscherRecords();
+                searchForPictures(escherRecords, pictures);
+            }
+        }
+        return pictures;
+    }
+
+    /**
+     * Performs a recursive search for pictures in the given list of escher records.
+     *
+     * @param escherRecords the escher records.
+     * @param pictures the list to populate with the pictures.
+     */
+    private void searchForPictures(List escherRecords, List pictures)
+    {
+        Iterator recordIter = escherRecords.iterator();
+        while (recordIter.hasNext())
+        {
+            Object obj = recordIter.next();
+            if (obj instanceof EscherRecord)
+            {
+                EscherRecord escherRecord = (EscherRecord) obj;
+
+                if (escherRecord instanceof EscherBSERecord)
+                {
+                    EscherBlipRecord blip = ((EscherBSERecord) escherRecord).getBlipRecord();
+                    if (blip != null)
+                    {
+                        // TODO: Some kind of structure.
+                        pictures.add(new HSSFPictureData(blip));
+                    }
+                }
+
+                // Recursive call.
+                searchForPictures(escherRecord.getChildRecords(), pictures);
+            }
+        }
+    }
+
+    /**
+     * protect a workbook with a password (not encypted, just sets writeprotect
+     * flags and the password.
+     * @param password to set
+     */
+    public void writeProtectWorkbook( String password, String username ) {
+       this.workbook.writeProtectWorkbook(password, username);
+    }
+
+    /**
+     * removes the write protect flag
+     */
+    public void unwriteProtectWorkbook() {
+       this.workbook.unwriteProtectWorkbook();
+    }
+
+    /**
+     * Gets all embedded OLE2 objects from the Workbook.
+     *
+     * @return the list of embedded objects (a list of {@link HSSFObjectData} objects.)
+     */
+    public List getAllEmbeddedObjects()
+    {
+        List objects = new ArrayList();
+        for (int i = 0; i < getNumberOfSheets(); i++)
+        {
+            getAllEmbeddedObjects(getSheetAt(i).getSheet().getRecords(), objects);
+        }
+        return objects;
+    }
+
+    /**
+     * Gets all embedded OLE2 objects from the Workbook.
+     *
+     * @param records the list of records to search.
+     * @param objects the list of embedded objects to populate.
+     */
+    private void getAllEmbeddedObjects(List records, List objects)
+    {
+        Iterator recordIter = records.iterator();
+        while (recordIter.hasNext())
+        {
+            Object obj = recordIter.next();
+            if (obj instanceof ObjRecord)
+            {
+                // TODO: More convenient way of determining if there is stored binary.
+                // TODO: Link to the data stored in the other stream.
+                Iterator subRecordIter = ((ObjRecord) obj).getSubRecords().iterator();
+                while (subRecordIter.hasNext())
+                {
+                    Object sub = subRecordIter.next();
+                    if (sub instanceof EmbeddedObjectRefSubRecord)
+                    {
+                        objects.add(new HSSFObjectData((ObjRecord) obj, filesystem));
+                    }
+                }
+            }
+        }
+    }
+
+    private byte[] newUID()
+    {
+        byte[] bytes = new byte[16];
+        return bytes;
+    }
 }
