@@ -1,9 +1,10 @@
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -183,7 +184,7 @@ public class HSSFColor
         hash.put(HSSFColor.YELLOW.hexString, new HSSFColor.YELLOW());
         hash.put(HSSFColor.BRIGHT_GREEN.hexString,
                  new HSSFColor.BRIGHT_GREEN());
-        hash.put(HSSFColor.BRIGHT_GREEN.hexString, new HSSFColor.TURQUOISE());
+        hash.put(HSSFColor.TURQUOISE.hexString, new HSSFColor.TURQUOISE());
         hash.put(HSSFColor.DARK_RED.hexString, new HSSFColor.DARK_RED());
         hash.put(HSSFColor.SKY_BLUE.hexString, new HSSFColor.SKY_BLUE());
         hash.put(HSSFColor.PLUM.hexString, new HSSFColor.PLUM());
@@ -1693,4 +1694,38 @@ public class HSSFColor
             return hexString;
         }
     }
+    
+    /**
+     * Special Default/Normal/Automatic color.
+     * <p><i>Note:</i> This class is NOT in the default HashTables returned by HSSFColor.
+     * The index is a special case which is interpreted in the various setXXXColor calls.
+     * 
+     * @author Jason
+     *
+     */
+    public final static class AUTOMATIC extends HSSFColor
+    {
+    	private static HSSFColor instance = new AUTOMATIC();
+    	
+        public final static short   index     = 0x40;
+
+        public short getIndex()
+        {
+            return index;
+        }
+
+        public short [] getTriplet()
+        {
+            return BLACK.triplet;
+        }
+
+        public String getHexString()
+        {
+            return BLACK.hexString;
+        }
+        
+        public static HSSFColor getInstance() {
+          return instance;
+        }
+    }
 }
