@@ -1,10 +1,11 @@
 
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -24,12 +25,8 @@ import org.apache.poi.hdf.extractor.util.*;
 import org.apache.poi.hdf.extractor.data.*;
 import java.util.*;
 import java.io.*;
-import javax.swing.*;
-
-import java.awt.*;
 
 import org.apache.poi.poifs.filesystem.POIFSFileSystem;
-import org.apache.poi.poifs.filesystem.POIFSDocument;
 import org.apache.poi.poifs.filesystem.DocumentEntry;
 
 import org.apache.poi.util.LittleEndian;
@@ -819,10 +816,14 @@ public class WordDocument
     int marginLeft = sep._dxaLeft/20;
     int marginRight = sep._dxaRight/20;
 
-    return "<fo:region-" + where + " display-align=\"" + align + "\" extent=\"" +
-             extent + "pt\" padding-left=\"" + marginLeft + "pt\" padding-right=\"" +
-             marginRight + "pt\" padding-top=\"" + marginTop + "pt\" padding-bottom=\"" +
-             marginBottom + "pt\" " + region + "/>";
+    return "<fo:region-" + where + " display-align=\"" + align + "\" extent=\""
+             + extent + "pt\" "+region+"/>";
+// org.apache.fop.fo.expr.PropertyException: 
+// Border and padding for region "xsl-region-before" must be '0' 
+// (See 6.4.13 in XSL 1.0).
+//             extent + "pt\" padding-left=\"" + marginLeft + "pt\" padding-right=\"" +
+//             marginRight + "pt\" padding-top=\"" + marginTop + "pt\" padding-bottom=\"" +
+//             marginBottom + "pt\" " + region + "/>";
 
   }
   private String createRegion(String where, String name)
