@@ -1,10 +1,11 @@
 
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -225,7 +226,7 @@ public class HSSFChart
             (byte) 0x87, (byte) 0x03, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
         };
 
-        return new UnknownRecord( (short) 0x005D, (short) 0x001a, data );
+        return new UnknownRecord( (short) 0x005D, data );
     }
 
     private UnknownRecord createMSDrawingObjectRecord()
@@ -249,7 +250,7 @@ public class HSSFChart
             (byte)0x00, (byte)0x00, (byte)0x11, (byte)0xF0, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00
         };
 
-        return new UnknownRecord((short)0x00EC, (short)0x00C8, data);
+        return new UnknownRecord((short)0x00EC, data);
     }
 
     private void createAxisRecords( List records )
