@@ -1,10 +1,11 @@
 
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -27,6 +28,7 @@ import java.util.Date;
 import junit.framework.TestCase;
 
 import org.apache.poi.hssf.util.CellReference;
+import org.apache.poi.util.TempFile;
 
 /**
  * @author Andrew C. Oliver (acoliver at apache dot org)
@@ -47,7 +49,7 @@ extends TestCase {
     throws Exception {
         
         short            rownum = 0;
-        File file = File.createTempFile("testFormula",".xls");
+        File file = TempFile.createTempFile("testFormula",".xls");
         FileOutputStream out    = new FileOutputStream(file);
         HSSFWorkbook     wb     = new HSSFWorkbook();
         HSSFSheet        s      = wb.createSheet();
@@ -169,7 +171,7 @@ extends TestCase {
     private void floatTest(String operator)
     throws Exception {
         short            rownum = 0;
-        File file = File.createTempFile("testFormulaFloat",".xls");
+        File file = TempFile.createTempFile("testFormulaFloat",".xls");
         FileOutputStream out    = new FileOutputStream(file);
         HSSFWorkbook     wb     = new HSSFWorkbook();
         HSSFSheet        s      = wb.createSheet();
@@ -260,7 +262,7 @@ extends TestCase {
     
     private void operationRefTest(String operator) 
     throws Exception {
-        File file = File.createTempFile("testFormula",".xls");
+        File file = TempFile.createTempFile("testFormula",".xls");
         FileOutputStream out    = new FileOutputStream(file);
         HSSFWorkbook     wb     = new HSSFWorkbook();
         HSSFSheet        s      = wb.createSheet();
@@ -412,7 +414,7 @@ extends TestCase {
      */    
     private void orderTest(String formula) 
     throws Exception {
-        File file = File.createTempFile("testFormula",".xls");
+        File file = TempFile.createTempFile("testFormula",".xls");
         FileOutputStream out    = new FileOutputStream(file);
         HSSFWorkbook     wb     = new HSSFWorkbook();
         HSSFSheet        s      = wb.createSheet();
@@ -450,7 +452,7 @@ extends TestCase {
     private void binomialOperator(String operator)
     throws Exception {
         short            rownum = 0;
-        File file = File.createTempFile("testFormula",".xls");
+        File file = TempFile.createTempFile("testFormula",".xls");
         FileOutputStream out    = new FileOutputStream(file);
         HSSFWorkbook     wb     = new HSSFWorkbook();
         HSSFSheet        s      = wb.createSheet();
@@ -548,7 +550,7 @@ extends TestCase {
     throws Exception {
             
             short            rownum = 0;
-            File file = File.createTempFile("testFormulaAreaFunction"+function,".xls");
+            File file = TempFile.createTempFile("testFormulaAreaFunction"+function,".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet();
@@ -586,7 +588,7 @@ extends TestCase {
     throws Exception {
             
             short            rownum = 0;
-            File file = File.createTempFile("testFormulaArrayFunction"+function,".xls");
+            File file = TempFile.createTempFile("testFormulaArrayFunction"+function,".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet();
@@ -625,7 +627,7 @@ extends TestCase {
     throws Exception {
             
             short            rownum = 0;
-            File file = File.createTempFile("testFormulaAreaArrayFunction"+function,".xls");
+            File file = TempFile.createTempFile("testFormulaAreaArrayFunction"+function,".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet();
@@ -664,7 +666,7 @@ extends TestCase {
     
     
     public void testAbsRefs() throws Exception {
-            File file = File.createTempFile("testFormulaAbsRef",".xls");
+            File file = TempFile.createTempFile("testFormulaAbsRef",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet();
@@ -711,7 +713,7 @@ extends TestCase {
     {
         String filename = System.getProperty("HSSF.testdata.path");
 
-            File file = File.createTempFile("testSheetFormula",".xls");
+            File file = TempFile.createTempFile("testSheetFormula",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet("A");
@@ -743,7 +745,7 @@ extends TestCase {
     }
     
     public void testRVAoperands() throws Exception {
-         File file = File.createTempFile("testFormulaRVA",".xls");
+         File file = TempFile.createTempFile("testFormulaRVA",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet();
@@ -785,7 +787,7 @@ extends TestCase {
     {
         String readFilename = System.getProperty("HSSF.testdata.path");
 
-            File file = File.createTempFile("testStringFormula",".xls");
+            File file = TempFile.createTempFile("testStringFormula",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet("A");
@@ -818,7 +820,7 @@ extends TestCase {
         throws IOException
     {
 
-            File file = File.createTempFile("testLogicalFormula",".xls");
+            File file = TempFile.createTempFile("testLogicalFormula",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
             HSSFSheet        s      = wb.createSheet("A");
@@ -847,10 +849,10 @@ extends TestCase {
     {
         String readFilename = System.getProperty("HSSF.testdata.path");
 
-            File file = File.createTempFile("testDateFormula",".xls");
+            File file = TempFile.createTempFile("testDateFormula",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
-            HSSFSheet        s      = wb.createSheet("Sheet1");
+            HSSFSheet        s      = wb.createSheet("testSheet1");
             HSSFRow          r      = null;
             HSSFCell         c      = null;
 
@@ -885,10 +887,10 @@ extends TestCase {
     {
         String readFilename = System.getProperty("HSSF.testdata.path");
 
-            File file = File.createTempFile("testIfFormula",".xls");
+            File file = TempFile.createTempFile("testIfFormula",".xls");
             FileOutputStream out    = new FileOutputStream(file);
             HSSFWorkbook     wb     = new HSSFWorkbook();
-            HSSFSheet        s      = wb.createSheet("Sheet1");
+            HSSFSheet        s      = wb.createSheet("testSheet1");
             HSSFRow          r      = null;
             HSSFCell         c      = null;
             r = s.createRow((short)0);
@@ -922,10 +924,10 @@ extends TestCase {
             //assertTrue("expected: A!A1+A!B1 got: "+c.getCellFormula(), ("A!A1+A!B1").equals(c.getCellFormula()));
             in.close();
             
-		File simpleIf = File.createTempFile("testSimpleIfFormulaWrite",".xls");
+		File simpleIf = TempFile.createTempFile("testSimpleIfFormulaWrite",".xls");
 		out    = new FileOutputStream(simpleIf);
 		wb     = new HSSFWorkbook();
-		s      = wb.createSheet("Sheet1");
+		s      = wb.createSheet("testSheet1");
 		r      = null;
 		c      = null;
 		r = s.createRow((short)0);
@@ -937,10 +939,10 @@ extends TestCase {
 			
 		assertTrue("length of simpleIf file is zero", (simpleIf.length()>0));
 			
-		File nestedIf = File.createTempFile("testNestedIfFormula",".xls");
+		File nestedIf = TempFile.createTempFile("testNestedIfFormula",".xls");
 		out    = new FileOutputStream(nestedIf);
 		wb     = new HSSFWorkbook();
-		s      = wb.createSheet("Sheet1");
+		s      = wb.createSheet("testSheet1");
 		r      = null;
 		c      = null;
 		r = s.createRow((short)0);
@@ -987,7 +989,7 @@ extends TestCase {
 		assertEquals(function, c.getCellFormula());
 		
 
-		File file = File.createTempFile("testSumIfFormula",".xls");
+		File file = TempFile.createTempFile("testSumIfFormula",".xls");
 		FileOutputStream out    = new FileOutputStream(file);
 		wb     = new HSSFWorkbook();
 		s      = wb.createSheet();
@@ -1083,6 +1085,60 @@ extends TestCase {
         assertEquals("String Cell value","XYZ",c.getStringCellValue());
     }
     
+    /** test for bug 34021*/
+    public void testComplexSheetRefs () throws IOException {
+    	HSSFWorkbook sb = new HSSFWorkbook();
+    	 HSSFSheet s1 = sb.createSheet("Sheet a.1");
+    	 HSSFSheet s2 = sb.createSheet("Sheet.A");
+    	 s2.createRow(1).createCell((short) 2).setCellFormula("'Sheet a.1'!A1");
+    	 s1.createRow(1).createCell((short) 2).setCellFormula("'Sheet.A'!A1");
+    	 File file = TempFile.createTempFile("testComplexSheetRefs",".xls");
+    	 sb.write(new FileOutputStream(file));
+    }
+    
+	/*Unknown Ptg 3C*/
+	public void test27272_1() throws Exception {
+		String readFilename = System.getProperty("HSSF.testdata.path");	
+		File inFile = new File(readFilename+"/27272_1.xls");
+		FileInputStream in = new FileInputStream(inFile);
+		HSSFWorkbook wb = new HSSFWorkbook(in);
+		wb.getSheetAt(0);
+		assertEquals("Reference for named range ", "#REF!",wb.getNameAt(0).getReference());
+		File outF = File.createTempFile("bug27272_1",".xls");
+		wb.write(new FileOutputStream(outF));
+		System.out.println("Open "+outF.getAbsolutePath()+" in Excel");
+	}
+	/*Unknown Ptg 3D*/
+	public void test27272_2() throws Exception {
+		String readFilename = System.getProperty("HSSF.testdata.path");	
+		File inFile = new File(readFilename+"/27272_2.xls");
+		FileInputStream in = new FileInputStream(inFile);
+		HSSFWorkbook wb = new HSSFWorkbook(in);
+		assertEquals("Reference for named range ", "#REF!",wb.getNameAt(0).getReference());
+		File outF = File.createTempFile("bug27272_2",".xls");
+		wb.write(new FileOutputStream(outF));
+		System.out.println("Open "+outF.getAbsolutePath()+" in Excel");
+	}
+	
+	/* MissingArgPtg */
+	public void testMissingArgPtg() throws Exception {
+		HSSFWorkbook wb = new HSSFWorkbook();
+		HSSFCell cell = wb.createSheet("Sheet1").createRow(4).createCell((short) 0);
+		cell.setCellFormula("IF(A1=\"A\",1,)");
+	}
+    
+	public void testSharedFormula() throws Exception {
+		String readFilename = System.getProperty("HSSF.testdata.path");	
+		File inFile = new File(readFilename+"/SharedFormulaTest.xls");
+		FileInputStream fis = new FileInputStream(inFile);
+		HSSFWorkbook wb = new HSSFWorkbook(fis);
+		
+		assertEquals("A$1*2", wb.getSheetAt(0).getRow(1).getCell((short)1).toString());
+		assertEquals("$A11*2", wb.getSheetAt(0).getRow(11).getCell((short)1).toString());
+		assertEquals("DZ2*2", wb.getSheetAt(0).getRow(1).getCell((short)128).toString());
+		assertEquals("B32770*2", wb.getSheetAt(0).getRow(32768).getCell((short)1).toString());
+	}
+    
     public static void main(String [] args) {
         System.out
         .println("Testing org.apache.poi.hssf.usermodel.TestFormulas");
