@@ -1,10 +1,11 @@
 
 /* ====================================================================
-   Copyright 2002-2004   Apache Software Foundation
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -18,21 +19,27 @@
 
 package org.apache.poi.hssf.usermodel;
 
+import junit.framework.TestCase;
+
+import org.apache.poi.hssf.util.AreaReference;
+import org.apache.poi.hssf.util.CellReference;
+import org.apache.poi.poifs.filesystem.POIFSFileSystem;
+import org.apache.poi.util.TempFile;
+
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 
-import junit.framework.TestCase;
-
-import org.apache.poi.poifs.filesystem.POIFSFileSystem;
-
 
 /**
  * 
  * @author ROMANL
  * @author Andrew C. Oliver (acoliver at apache dot org)
  * @author Danny Mui (danny at muibros.com)
+ * @author Amol S. Deshmukh &lt; amol at ap ache dot org &gt; 
  */
 public class TestNamedRange
     extends TestCase {
@@ -105,7 +112,7 @@ public class TestNamedRange
         SanityChecker c = new SanityChecker();
         c.checkHSSFWorkbook(wb);
 
-        File             file = File.createTempFile("testNamedRange",
+        File             file = TempFile.createTempFile("testNamedRange",
                                         ".xls");
 
         FileOutputStream fileOut = new FileOutputStream(file);
@@ -127,7 +134,7 @@ public class TestNamedRange
     /**
      * Reads an excel file already containing a named range.
      * <p>
-	 * Addresses Bug <a href="http://nagoya.apache.org/bugzilla/show_bug.cgi?id=9632" target="_bug">#9632</a>
+	 * Addresses Bug <a href="http://issues.apache.org/bugzilla/show_bug.cgi?id=9632" target="_bug">#9632</a>
      */   
 	public void testNamedRead() throws IOException
 	{        
@@ -168,7 +175,7 @@ public class TestNamedRange
     /**
      * Reads an excel file already containing a named range and updates it
      * <p>
-     * Addresses Bug <a href="http://nagoya.apache.org/bugzilla/show_bug.cgi?id=16411" target="_bug">#16411</a>
+     * Addresses Bug <a href="http://issues.apache.org/bugzilla/show_bug.cgi?id=16411" target="_bug">#16411</a>
      */
     public void testNamedReadModify() throws IOException
     {
@@ -208,10 +215,10 @@ public class TestNamedRange
 	    HSSFWorkbook wb     = new HSSFWorkbook();
          
 
-		HSSFSheet sheet = wb.createSheet("Sheet1");
+		HSSFSheet sheet = wb.createSheet("testSheet1");
 		String sheetName = wb.getSheetName(0);
         
-        assertEquals("Sheet1", sheetName);
+        assertEquals("testSheet1", sheetName);
          
 		//Creating new Named Range
 		HSSFName newNamedRange = wb.createName();
@@ -229,7 +236,7 @@ public class TestNamedRange
 		HSSFName namedRange1 = wb.getNameAt(0);
 		String referece = namedRange1.getReference();
                                 
-	    File file = File.createTempFile("testMultiNamedRange", ".xls");
+	    File file = TempFile.createTempFile("testMultiNamedRange", ".xls");
  
 		FileOutputStream fileOut = new FileOutputStream(file);
 		wb.write(fileOut);
@@ -255,7 +262,7 @@ public class TestNamedRange
 	/**
 	 * Test case provided by czhang@cambian.com (Chun Zhang)
 	 * <p>
-	 * Addresses Bug <a href="http://nagoya.apache.org/bugzilla/show_bug.cgi?id=13775" target="_bug">#13775</a>
+	 * Addresses Bug <a href="http://issues.apache.org/bugzilla/show_bug.cgi?id=13775" target="_bug">#13775</a>
 	 * @throws IOException
 	 */
 	public void testMultiNamedRange() 
@@ -293,7 +300,7 @@ public class TestNamedRange
 		 namedRange2.setReference("sheet2" + "!$A$1:$O$21");        
  
 		 // Write the workbook to a file
-		 File file = File.createTempFile("testMuiltipletNamedRanges", ".xls");
+		 File file = TempFile.createTempFile("testMuiltipletNamedRanges", ".xls");
 		 FileOutputStream fileOut = new FileOutputStream(file);
 		 wb.write(fileOut);
 		 fileOut.close();
@@ -311,7 +318,23 @@ public class TestNamedRange
 		 assertTrue("Name is "+nm2.getNameName(),"RangeTest2".equals(nm2.getNameName()));
 		 assertTrue("Reference is "+nm2.getReference(),(wb.getSheetName(1)+"!$A$1:$O$21").equals(nm2.getReference()));
 	 }       
-        
+
+	public void testUnicodeNamedRange() throws Exception {
+	    HSSFWorkbook workBook = new HSSFWorkbook();
+	    HSSFSheet sheet = workBook.createSheet("Test");
+	    HSSFName name = workBook.createName();
+	    name.setNameName("\u03B1");
+	    name.setReference("Test!$D$3:$E$8");
+	    
+	    ByteArrayOutputStream out = new ByteArrayOutputStream();
+	    workBook.write(out);
+	    
+	    HSSFWorkbook workBook2 = new HSSFWorkbook(new ByteArrayInputStream(out.toByteArray()));
+	    HSSFName name2 = workBook2.getNameAt(0);
+	    
+	    assertEquals("\u03B1", name2.getNameName());
+	    assertEquals("Test!$D$3:$E$8", name2.getReference());
+	}
         
 	 /**
 	  * Test to see if the print areas can be retrieved/created in memory
@@ -401,7 +424,7 @@ public class TestNamedRange
 	 	String reference = sheetName+"!$A$1:$B$1";
 	 	workbook.setPrintArea(0, reference);
 	     
-	    File file = File.createTempFile("testPrintArea",".xls");        
+	    File file = TempFile.createTempFile("testPrintArea",".xls");
 	     
 	    FileOutputStream fileOut = new FileOutputStream(file);
 	    workbook.write(fileOut);
@@ -444,7 +467,7 @@ public class TestNamedRange
 		String reference3 = sheetName+"!$D$2:$F$5";
 		workbook.setPrintArea(2, reference3);
 	    
-	    File file = File.createTempFile("testMultiPrintArea",".xls");        
+	    File file = TempFile.createTempFile("testMultiPrintArea",".xls");        
 	    
 	    FileOutputStream fileOut = new FileOutputStream(file);
 	    workbook.write(fileOut);
@@ -507,6 +530,78 @@ public class TestNamedRange
     	workbook.removePrintArea(0);
     	assertNull("PrintArea was not removed", workbook.getPrintArea(0)); 
     }
+    
+    /**
+     * Verifies correct functioning for "single cell named range" (aka "named cell")
+     */
+    public void testNamedCell_1() {
         
+        // setup for this testcase
+        String sheetName = "Test Named Cell";
+        String cellName = "A name for a named cell";
+        String cellValue = "TEST Value";
+        HSSFWorkbook wb = new HSSFWorkbook();        
+        HSSFSheet sheet = wb.createSheet(sheetName);
+        sheet.createRow(0).createCell((short) 0).setCellValue(cellValue);
+         
+        // create named range for a single cell using areareference
+        HSSFName namedCell = wb.createName();
+        namedCell.setNameName(cellName);
+        String reference = sheetName+"!A1:A1";
+        namedCell.setReference(reference);
+        
+        // retrieve the newly created named range
+        int namedCellIdx = wb.getNameIndex(cellName);
+        HSSFName aNamedCell = wb.getNameAt(namedCellIdx);
+        assertNotNull(aNamedCell);
+        
+        // retrieve the cell at the named range and test its contents
+        AreaReference aref = new AreaReference(aNamedCell.getReference());
+        CellReference[] crefs = aref.getCells();
+        assertNotNull(crefs);
+        assertEquals("Should be exactly 1 cell in the named cell :'" +cellName+"'", 1, crefs.length);
+        for (int i=0, iSize=crefs.length; i<iSize; i++) {
+            CellReference cref = crefs[i];
+            assertNotNull(cref);
+            HSSFSheet s = wb.getSheet(cref.getSheetName());
+            HSSFRow r = sheet.getRow(cref.getRow());
+            HSSFCell c = r.getCell(cref.getCol());
+            String contents = c.getStringCellValue();
+            assertEquals("Contents of cell retrieved by its named reference", contents, cellValue);
+        }
+    }
+
+    /**
+     * Verifies correct functioning for "single cell named range" (aka "named cell")
+     */
+    public void testNamedCell_2() {
+        
+        // setup for this testcase
+        String sname = "TestSheet", cname = "TestName", cvalue = "TestVal";
+        HSSFWorkbook wb = new HSSFWorkbook();
+        HSSFSheet sheet = wb.createSheet(sname);
+        sheet.createRow(0).createCell((short) 0).setCellValue(cvalue);
+         
+        // create named range for a single cell using cellreference
+        HSSFName namedCell = wb.createName();
+        namedCell.setNameName(cname);
+        String reference = sname+"!A1";
+        namedCell.setReference(reference);
+        
+        // retrieve the newly created named range
+        int namedCellIdx = wb.getNameIndex(cname);
+        HSSFName aNamedCell = wb.getNameAt(namedCellIdx);
+        assertNotNull(aNamedCell);
+        
+        // retrieve the cell at the named range and test its contents
+        CellReference cref = new CellReference(aNamedCell.getReference());
+        assertNotNull(cref);
+        HSSFSheet s = wb.getSheet(cref.getSheetName());
+        HSSFRow r = sheet.getRow(cref.getRow());
+        HSSFCell c = r.getCell(cref.getCol());
+        String contents = c.getStringCellValue();
+        assertEquals("Contents of cell retrieved by its named reference", contents, cvalue);
+    }
+
 }
 
